(*====================================================================
    File rdbcomboedit.pas

    Components library for Delphi and BCB++.
    TrDBComboEdit.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rDBComboEdit.pas,v 1.5 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)
unit rDBComboEdit;

{$I POLARIS.INC}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  rUtils, rVCLUtils, RXDBCtrl, ToolEdit, DB {$IFDEF POLARIS_D6}, RxStrUtils, Variants{$ENDIF};

type
  TUpperCase = (ucNone, ucOnlyFirst{$IFNDEF LINUX}, ucInitials{$ENDIF});

  TrDBComboEdit = class(TRxDBComboEdit)
  private
    FGlyphResKind: TBitmapResKind;
    FGlyphResource: string;
    FOnCloseUp: TNotifyEvent;
    FEmptyIsNull: Boolean;
    FUpperCase: TUpperCase;
    FTrimBlanks: Boolean;
    function IsGlyph: Boolean;
    procedure SetGlyphResKind(Value: TBitmapResKind);
    procedure SetGlyphResource(Value: string);
{$IFNDEF LINUX}
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
{$ENDIF}
  protected
{$IFDEF LINUX}
    procedure DoExit; override;
{$ENDIF}
    property OnCloseUp: TNotifyEvent read FOnCloseUp write FOnCloseUp;
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    property Button;
  published
    property Align;
    property Glyph stored IsGlyph;
    property GlyphResKind:  TBitmapResKind read FGlyphResKind write SetGlyphResKind
                                           default DefBRK;
    property GlyphResource: string read FGlyphResource write SetGlyphResource;
    property EmptyIsNull: Boolean read FEmptyIsNull write FEmptyIsNull default False;
    property UpperCase: TUpperCase read FUpperCase write FUpperCase default ucNone;
    property TrimBlanks: Boolean read FTrimBlanks write FTrimBlanks default False;
  end;

implementation

{ TrDBComboEdit }

constructor TrDBComboEdit.Create(AOwner: TComponent);
begin
  inherited;
  FGlyphResKind := DefBRK;
  FUpperCase := ucNone;
  FTrimBlanks := False;
end;

function TrDBComboEdit.IsGlyph;
begin
  Result := ((FGlyphResKind = rkNone) or (Trim(FGlyphResource) = '')) and (GlyphKind = gkCustom);
end;

procedure TrDBComboEdit.SetGlyphResource;
begin
  if FGlyphResource <> Value then begin
    FGlyphResource := Value;
    UpdateGlyph(Self, Button.Glyph, FGlyphResKind, FGlyphResource);
  end;
end;

procedure TrDBComboEdit.SetGlyphResKind;
begin
  if FGlyphResKind <> Value then begin
    FGlyphResKind := Value;
    UpdateGlyph(Self, Button.Glyph, FGlyphResKind, FGlyphResource);
  end;
end;

procedure TrDBComboEdit.Loaded;
begin
  inherited;
  UpdateGlyph(Self, Button.Glyph, FGlyphResKind, FGlyphResource);
end;

{$IFDEF LINUX}
procedure TrDBComboEdit.DoExit;
{$ELSE}
procedure TrDBComboEdit.CMExit(var Message: TCMExit);
{$ENDIF}
begin
  if FTrimBlanks then Text := Trim(Text);
  if (Length(Text) <> 0) and (DataSource.State in dsEditModes) then begin
    if FUpperCase = ucOnlyFirst then
      Text := AnsiUpperCase(String(Text[1]))+Copy(Text,2,Length(Text))
{$IFNDEF LINUX}
    else
      if FUpperCase = ucInitials then
        Text := AnsiProperCase(Text,[' ','.',',','-']);
{$ENDIF}
  end;
  inherited;
  if FEmptyIsNull and (Length(Text)=0) then
    with DataSource do
      if State in dsEditModes then DataSet[DataField] := NULL;
end;

end.
