(*====================================================================
    File rdbedit.pas

    Components library for Delphi and BCB++.
    Descendant of TDBEdit.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rDBEdit.pas,v 1.4 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)

unit rDBEdit;

{$I POLARIS.INC}

interface

uses
  SysUtils, Classes,
{$IFDEF LINUX}
  QControls, QComCtrls, QMenus, QMask, QDBCtrls, DB, Variants;
{$ELSE}
  Windows, Messages, Graphics, Controls, Forms, Dialogs,
  ToolWin, ComCtrls, Menus, StdCtrls, Mask, DBCtrls, DB,
  {$IFDEF POLARIS_D6}RxStrUtils, Variants{$ELSE}StrUtils{$ENDIF};
{$ENDIF}

type
  TUpperCase = (ucNone, ucOnlyFirst{$IFNDEF LINUX}, ucInitials{$ENDIF});

  TrDBEdit = class(TDBEdit)
  private
    { Private declarations }
    FUpperCase: TUpperCase;
    FTrimBlanks: Boolean;
    FEmptyIsNull: Boolean;
{$IFNDEF LINUX}
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
{$ENDIF}
  protected
{$IFDEF LINUX}
    procedure DoExit; override;
{$ENDIF}
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property UpperCase: TUpperCase read FUpperCase write FUpperCase default ucOnlyFirst;
    property TrimBlanks: Boolean read FTrimBlanks write FTrimBlanks default True;
    property EmptyIsNull: Boolean read FEmptyIsNull write FEmptyIsNull default True;
  end;

implementation

constructor TrDBEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FUpperCase := ucOnlyFirst;
  FTrimBlanks := True;
  FEmptyIsNull := True;
end;

{$IFDEF LINUX}
procedure TrDBEdit.DoExit;
{$ELSE}
procedure TrDBEdit.CMExit(var Message: TCMExit);
{$ENDIF}
begin
  if not Modified then exit;
  if FTrimBlanks then Text := Trim(Text);
  if (Length(Text) <> 0) and (DataSource.State in dsEditModes) then begin
    if FUpperCase = ucOnlyFirst then
      Text := AnsiUpperCase(String(Text[1]))+Copy(Text,2,Length(Text))
{$IFNDEF LINUX}
    else
      if FUpperCase = ucInitials then
        Text := AnsiProperCase(Text,[' ','.',',','-']);
{$ENDIF}
  end;
  inherited;
  if FEmptyIsNull and (Length(Text)=0) then
    with DataSource do
      if State in dsEditModes then DataSet[DataField] := NULL;
end;

end.
