(*====================================================================
    File rDBTimeEdit.pas

    Components library for Delphi and BCB++.
    rDBTimeEdit component.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rDBTimeEdit.pas,v 1.5 2008/09/26 15:17:08 savchenko Exp $
======================================================================*)
unit rDBTimeEdit;

{$I POLARIS.INC}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, StdCtrls, Mask, RxSpin, rUtils,
  {$IFDEF POLARIS_D6}Variants,{$ENDIF} DB, DBCtrls, rTimeEdit;

type
  TrDBTimeEdit = class( TrTimeEdit )
  private
    FChanged: boolean;
    FReplaceTimeOnly: boolean;
    FDataLink: TFieldDataLink;
    procedure DataChange(Sender: TObject);
    procedure EditingChange(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure UpdateData(Sender: TObject);
    procedure SetReplaceTimeOnly( Value: boolean );
    //     Field-
    function GetFieldTime: string;
    //   
    //  (    )  
    procedure SetFieldValue( Value: Variant );
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
  protected
    // ,    text -   FIELD-
    //       FIELD-  TEXT
    function NeedUpdate: boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Change; override;
    property Field: TField read GetField;
  published
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property ReplaceTimeOnly: boolean read FReplaceTimeOnly write SetReplaceTimeOnly;
    property FieldTime: string read GetFieldTime;
  end;

implementation

{ TrDBTimeEdit }

procedure TrDBTimeEdit.Change;
begin
  if FDatalink = Nil then Exit;
  if FDatalink.Field = Nil then Exit;
  if not FDatalink.Field.Dataset.Active then Exit;
  if not (FDatalink.Field.DataSet.State in [dsEdit,dsInsert]) then
    begin
      if FieldTime <> Text then
{        if FDatalink.Field.DataSet.Bof and FDatalink.Field.DataSet.Eof then
          FDatalink.Field.DataSet.Insert else
          FDatalink.Field.DataSet.Edit;}
      Text := FieldTime;
    end;
  inherited;
//  UpdateData( Self );
end;

constructor TrDBTimeEdit.Create(AOwner: TComponent);
begin
  inherited;
  FChanged := false;
  FReplaceTimeOnly := true;
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnEditingChange := EditingChange;
  FDataLink.OnUpdateData := UpdateData;
end;

procedure TrDBTimeEdit.DataChange(Sender: TObject);
begin
  if FDatalink.Field = Nil then Exit;
  if Focused and (FDatalink.Field.Dataset.State in [dsInsert,dsEdit]) then Exit;
  Text := FieldTime;
end;

destructor TrDBTimeEdit.Destroy;
begin
  FDataLink.Free;
  FDataLink := nil;
  inherited;
end;

procedure TrDBTimeEdit.EditingChange(Sender: TObject);
begin
  //
end;


function TrDBTimeEdit.GetFieldTime: string;

  function WS( i: Word ): string;
  begin
    Result := IntToStr( i );
    if length( Result ) < 2 then Result := '0' + Result;
  end;

Var Hour, Min, Sec, MSec: Word;
begin
  if WithSeconds then Result := '  :  :  ' else Result := '  :  ';
  if FDatalink = Nil then Exit;
  if FDatalink.Field = Nil then Exit;
  if FDatalink.Field.IsNull then Exit;
  DecodeTime( FDatalink.Field.AsDateTime, Hour, Min, Sec, MSec );
  Result := WS( Hour ) + ':' + WS( Min );
  if WithSeconds then Result := Result + ':' + WS( Sec );
end;

function TrDBTimeEdit.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

function TrDBTimeEdit.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

function TrDBTimeEdit.GetField: TField;
begin
  Result := FDataLink.Field;
end;

procedure TrDBTimeEdit.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

procedure TrDBTimeEdit.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

procedure TrDBTimeEdit.SetReplaceTimeOnly(Value: boolean);
begin
  FReplaceTimeOnly := Value;
end;

procedure TrDBTimeEdit.UpdateData(Sender: TObject);
Var T: Variant;
begin
  if FDatalink = Nil then Exit;
  if FDatalink.Field = Nil then Exit;
  T := Time;
  //     
  if VarIsNull(T) then
    begin
      if FDatalink.Field.IsNull then Exit;
      if FReplaceTimeOnly then
        begin
          T := fRound( FDatalink.Field.AsDateTime, 0);
          if FDatalink.Field.AsDateTime <> T then SetFieldValue( T );
        end else
          SetFieldValue( Null );
      Exit;
    end;
  //    ,  
  if FieldTime = Text then Exit;
  if FReplaceTimeOnly then T := T + fRound( FDatalink.Field.AsDateTime, 0);
  SetFieldValue( T );
end;

procedure TrDBTimeEdit.SetFieldValue(Value: Variant);
Var LastSelStart: integer;
    LastSelLength: integer;
begin
  if FDatalink = Nil then Exit;
  if FDatalink.Field = Nil then Exit;
  //      
  if not (FDatalink.Field.Dataset.State in [dsInsert,dsEdit]) then
    begin
      if not FDatalink.Datasource.AutoEdit then
        begin
          Text := FieldTime;
          Exit;
        end;
      //      -
      LastSelStart := SelStart;
      LastSelLength := SelLength;
      if LastSelLength = 0 then LastSelLength := 1;
      try
        if FDatalink.Field.DataSet.Bof and FDatalink.Field.DataSet.Eof then
          FDatalink.Field.DataSet.Insert else FDatalink.Field.DataSet.Edit;
      finally
        if Focused then
          begin
            //SelStart := LastSelStart;
            //SelLength := LastSelLength;
            PostMessage( Handle, WM_GOSEL, LastSelStart, LastSelLength );
          end;
      end;    
    end;
  if VarIsNull( Value ) then FDatalink.Field.Clear else
                             FDatalink.Field.AsDateTime := Value;
end;


procedure TrDBTimeEdit.CMExit(var Message: TCMExit);
begin
  if (FDatalink.Field <> Nil) and
     (FDatalink.Field.DataSet.State in [dsInsert,dsEdit]) then
    UpdateData( Self );
end;

function TrDBTimeEdit.NeedUpdate: boolean;
begin
  //      ,   
  Result := true;
  if not Focused then Exit;
  if FDatalink = Nil then Exit;
  if FDatalink.Field = Nil then Exit;
end;

end.
