(*====================================================================
    File rmenubar.pas

    Components library for Delphi and BCB++.
    Menu as in Delphi 4,5.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rMenuBar.pas,v 1.2 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)
unit rMenuBar;

{$I POLARIS.INC}

interface

uses
  SysUtils, Classes,
  Windows, Messages, Graphics, Controls, Forms, Dialogs,
  ToolWin, ComCtrls, Menus;
type
  TrMenuBar = class(TToolBar)
  private
{$IFNDEF POLARIS_D6}
    FMenu: TMainMenu;
    procedure SetMenu(const Value: TMainMenu);
{$ENDIF}
    function HandleAppKeyDown(var Message: TWMKey): Boolean;
    procedure CMDialogKey(var Message: TWMKey); message CM_DIALOGKEY;
  protected
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
{$IFNDEF POLARIS_D6}
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
{$ENDIF}
    procedure ClearButtons;
  public
    constructor Create(AOwner: TComponent); override;
    procedure RefreshButtons;
  published
    property EdgeBorders default [];
{$IFNDEF POLARIS_D6}
    property Menu: TMainMenu read FMenu write SetMenu;
{$ENDIF}
  end;

implementation

{ TrMenuBar }

//      MenuItem-
procedure AssignMenuItem( Button: TToolButton; MenuItem: TMenuItem );
begin
  Button.Caption := MenuItem.Caption;
  Button.Down := MenuItem.Checked;
  Button.Enabled := MenuItem.Enabled;
  Button.Hint := MenuItem.Hint;
  Button.ImageIndex := MenuItem.ImageIndex;
  Button.Visible := MenuItem.Visible;
  Button.Action := MenuItem.Action;
end;


constructor TrMenuBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Flat := True;
  ShowCaptions := True;
  EdgeBorders := [];
  ControlStyle := [csCaptureMouse, csClickEvents,
    csDoubleClicks, csMenuEvents, csSetCaption];
  RecreateWnd;
end;

procedure TrMenuBar.CMDialogKey(var Message: TWMKey);
begin
  if not HandleAppKeyDown(Message) then Inherited;
end;

function TrMenuBar.HandleAppKeyDown(var Message: TWMKey): Boolean;
begin
  if Assigned(Menu) and Menu.IsShortCut(Message) then begin
    Message.Result := 1;
    Result := True;
  end else
    Result := False;
end;

procedure TrMenuBar.ClearButtons;
var
  i: Integer;
begin
  for i := ButtonCount - 1 downto 0 do Buttons[i].Free;
end;

procedure TrMenuBar.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
end;

{$IFNDEF POLARIS_D6}
procedure TrMenuBar.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FMenu)
  then begin
    ClearButtons;
    FMenu := nil;
  end;
end;

procedure TrMenuBar.SetMenu(const Value: TMainMenu);
var
  i: Integer;
begin
  if FMenu = Value then exit;
  if Assigned(FMenu) then ClearButtons;
  FMenu := Value;
  if not Assigned(FMenu) then exit;
  Value.FreeNotification(Self);
  for i := 0 to FMenu.Items.Count - 1 do
    with TToolButton.Create(Self) do
      try
        AutoSize := True;
        Grouped := True;
        Parent := Self;
      except
        Free;
        raise;
      end;
  for i := 0 to ButtonCount - 1 do Buttons[i].MenuItem := FMenu.Items[i];
end;
{$ENDIF}

procedure TrMenuBar.RefreshButtons;
var
  i: Integer;
begin
  for i := 0 to ButtonCount - 1 do
    AssignMenuItem( Buttons[i], Buttons[i].MenuItem );
  ////with Buttons[i] do Enabled := MenuItem.Enabled;
end;

end.
