(*====================================================================
    File rSQLQuery.pas

    Components library for Delphi and BCB++.
    rSQLQuery component.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rSQLQuery.pas,v 1.5 2008/10/06 11:45:47 khmelevskiy Exp $
======================================================================*)
unit rSQLQuery;

{$I POLARIS.INC}

{$ifndef mswindows}
  {$undef RSQLMONITOR}
{$endif}

interface

uses
  {$ifdef mswindows} Windows, {$endif}
  SysUtils, Classes, Variants, DB, SqlConst, DBXpress, SqlExpr, RxStrUtils, rUtils
  {$IFDEF RSQLMONITOR} ,DDEMan {$ENDIF}
  ;

const
  DefaultMacroChar = '%';
  DefaultTermChar  = '/';
  TrueExpr         = '0=0';
  cDelimiter = #4;

//  ,        DDE-
// (   SQL  )
const NoPassMessages: array[0..7] of string =
 ('INTERBASE - isc_dsql_allocate_statement',
  'INTERBASE - isc_dsql_free_statement',
  'INTERBASE - isc_close_blob',
  'INTERBASE - isc_blob_info',
  'INTERBASE - isc_open_blob',
  'INTERBASE - isc_get_segment',
  'INTERBASE - isc_dsql_fetch',
  'INTERBASE - isc_dsql_describe_bind' );

type

  TrSQLConnection = class(TSQLConnection)
  private
    FVendorVersion: Variant;
    function GetVendorVersion: string;
  protected
    procedure DoDisconnect; override;
    procedure ConnectionOptions; override;
  public
    procedure AfterConstruction; override;
    property VendorVersion: string read GetVendorVersion;
  end;

  TrSQLQuery = class(TSQLQuery)
  private
    {$IFDEF RSQLMONITOR}
    FDDE: TDDEClientConv;
    {$ENDIF}
    FSQL: TStrings;
    FText: string;
    FMacroChar: Char;
    FMacros: TParams;
    function  GetMacros: TParams;
    procedure SetMacros(Value: TParams);
    function  GetMacroCount: Word;
    procedure SetMacroChar(Value: Char);
    procedure RecreateMacros;
    procedure CreateMacros(List: TParams; const Value: PChar);
    procedure QueryChanged(Sender: TObject);
    procedure SetSQL(Value: TStrings);
    function GetRealSQL: string;
    function GetPrepared: boolean;
    procedure SetPrepared( Value: boolean );
  protected
    procedure DoBeforeOpen; override;
    procedure InternalOpen; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ExecSQL(ExecDirect: Boolean = False): Integer; override;
    function MacroByName(const Value: string): TParam;
    procedure PrepareStatement; override;
    property RowsAffected;
    property Text: string read FText;
    property MacroCount: Word read GetMacroCount;
    property RealSQL:string read GetRealSQL;
    property Prepared: boolean read GetPrepared write SetPrepared;
  published
    property DataSource;
    property MacroChar: Char read FMacroChar write SetMacroChar default DefaultMacroChar;
    property Macros: TParams read GetMacros write SetMacros;
    property MaxBlobSize;
    property ParamCheck;
    property Params;
    property SQL: TStrings read FSQL write SetSQL;
  end;

  TrSQLStoredProc = class(TSQLStoredProc)
  private
    {$IFDEF RSQLMONITOR}
    FDDE: TDDEClientConv;
    {$ENDIF}
  public
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
    function ExecProc: Integer; override;
    procedure PrepareStatement; override;
  end;


procedure CreateQueryParams(List: TParams; const Value: PChar; Macro: Boolean;
  SpecialChar: Char; Delims: TSysCharSet);
function NameDelimiter(C: Char; Delims: TSysCharSet): Boolean;
function IsLiteral(C: Char): Boolean;

{$IFDEF RSQLMONITOR}
//     SQL Monitor,   NIL
function CheckDDE: TDDEClientConv;

//  ,     SQL - 
function TextToSQLMonitor( Q: TrSQLQuery; const Prefix: string ): string; overload;
//  ,     SQL - 
function TextToSQLMonitor( Q: TrSQLStoredProc; const Prefix: string ): string; overload;

//   SQL Connection  
procedure PrepareToMonitoring( SQLConnection: TSQLConnection );

procedure FreeMonitoring;

function IsBad( const S: string ): boolean;
{$ENDIF}

implementation

{$IFNDEF POLARIS_D12}
const
  coLongStrings         = TSQLConnectionOption(101);
  coEnableBCD           = TSQLConnectionOption(102);
  coTrimFixedChar       = TSQLConnectionOption(103);
  coInternalName        = TSQLConnectionOption(201);
  coUseQuoteChar        = TSQLConnectionOption(202);
  coCharLength          = TSQLConnectionOption(203);
  coCharset             = TSQLConnectionOption(204);
  coIntegerPrecision    = TSQLConnectionOption(205);
  coSmallIntPrecision   = TSQLConnectionOption(206);
  coFloatPrecision      = TSQLConnectionOption(207);
  coBCDPrecision        = TSQLConnectionOption(208);
  coFetchAll            = TSQLConnectionOption(301);
  coPrepared            = TSQLConnectionOption(302);
  coParamPrefix         = TSQLConnectionOption(303);
  coEnableBoolean       = TSQLConnectionOption(304);

  SLongStrings          = 'LongStrings';
  SEnableBCD            = 'EnableBCD';
  SInternalName         = 'InternalName';
  SUseQuoteChar         = 'UseQuoteChar';
  SFetchAll             = 'FetchAll';
  SCharLength           = 'CharLength';
  SPrepared             = 'Prepared';
  SParamPrefix          = 'ParamPrefix';
  STrimFixedChar        = 'TrimFixedChar';
  SCharset              = 'Charset';
  SIntegerPrecision     = 'IntegerPrecision';
  SSmallIntPrecision    = 'SmallIntPrecision';
  SFloatPrecision       = 'FloatPrecision';
  SBCDPrecision         = 'BCDPrecision';
  SEnableBoolean        = 'EnableBoolean';
{$ENDIF}

{$IFDEF RSQLMONITOR}
function IsBad( const S: string ): boolean;
var i: integer;
begin
  Result := false;
  for i := Low( NoPassMessages ) to High( NoPassMessages ) do
    if NoPassMessages[i] = copy(S,1,length(NoPassMessages[i])) then
      begin
        Result := true;
        Break;
      end;
end;
{$ENDIF}

function NameDelimiter(C: Char; Delims: TSysCharSet): Boolean;
begin
  Result := (C in [' ', ',', ';', ')', #13, #10]) or (C in Delims);
end;

function IsLiteral(C: Char): Boolean;
begin
  Result := C in ['''', '"'];
end;

procedure CreateQueryParams(List: TParams; const Value: PChar; Macro: Boolean;
  SpecialChar: Char; Delims: TSysCharSet);
var
  CurPos, StartPos: PChar;
  CurChar: Char;
  Literal: Boolean;
  EmbeddedLiteral: Boolean;
  Name: string;

  function StripLiterals(Buffer: PChar): string;
  var
    Len: Word;
    TempBuf: PChar;

    procedure StripChar(Value: Char);
    begin
      if TempBuf^ = Value then
        StrMove(TempBuf, TempBuf + 1, Len - 1);
      if TempBuf[StrLen(TempBuf) - 1] = Value then
        TempBuf[StrLen(TempBuf) - 1] := #0;
    end;

  begin
    Len := StrLen(Buffer) + 1;
    TempBuf := AllocMem(Len);
    Result := '';
    try
      StrCopy(TempBuf, Buffer);
      StripChar('''');
      StripChar('"');
      Result := StrPas(TempBuf);
    finally
      FreeMem(TempBuf, Len);
    end;
  end;

begin
  if SpecialChar = #0 then Exit;
  CurPos := Value;
  Literal := False;
  EmbeddedLiteral := False;
  repeat
    CurChar := CurPos^;
    if (CurChar = SpecialChar) and not Literal and ((CurPos + 1)^ <> SpecialChar) then
    begin
      StartPos := CurPos;
      while (CurChar <> #0) and (Literal or not NameDelimiter(CurChar, Delims)) do begin
        Inc(CurPos);
        CurChar := CurPos^;
        if IsLiteral(CurChar) then begin
          Literal := Literal xor True;
          if CurPos = StartPos + 1 then EmbeddedLiteral := True;
        end;
      end;
      CurPos^ := #0;
      if EmbeddedLiteral then begin
        Name := StripLiterals(StartPos + 1);
        EmbeddedLiteral := False;
      end
      else Name := StrPas(StartPos + 1);
      if Assigned(List) then begin
          if Macro then
            List.CreateParam(ftString, Name, ptInput).AsString := TrueExpr
          else List.CreateParam(ftUnknown, Name, ptUnknown);
      end;
      CurPos^ := CurChar;
      StartPos^ := '?';
      Inc(StartPos);
      StrMove(StartPos, CurPos, StrLen(CurPos) + 1);
      CurPos := StartPos;
    end
    else if (CurChar = SpecialChar) and not Literal and ((CurPos + 1)^ = SpecialChar) then
      StrMove(CurPos, CurPos + 1, StrLen(CurPos) + 1)
    else if IsLiteral(CurChar) then Literal := Literal xor True;
    Inc(CurPos);
  until CurChar = #0;
end;

{ TrSQLConnection }

procedure TrSQLConnection.AfterConstruction;
begin
  inherited;
  FVendorVersion := NULL;
end;

procedure TrSQLConnection.DoDisconnect;
begin
  inherited;
  FVendorVersion := NULL;
end;

procedure TrSQLConnection.ConnectionOptions;
begin
//copy from CoreLab CRSQLConnection component

  {$IFDEF POLARIS_D12}
  inherited;
  //  DBXConnection.ConnectionProperties?   trial .
  {$ELSE}
  if Params.Values[SUseQuoteChar] <> '' then
    SQLConnection.SetOption(coUseQuoteChar, LongInt(UpperCase(trim(Params.Values[SUseQuoteChar])) = 'TRUE'));

  inherited;

  if Params.Values[SLongStrings] <> '' then
    SQLConnection.SetOption(coLongStrings, LongInt(UpperCase(trim(Params.Values[SLongStrings])) = 'TRUE'));
  if Params.Values[SEnableBCD] <> '' then
    SQLConnection.SetOption(coEnableBCD, LongInt(UpperCase(trim(Params.Values[SEnableBCD])) = 'TRUE'));
  if Params.Values[SInternalName] <> '' then
    SQLConnection.SetOption(coInternalName, Integer(Params.Values[SInternalName]));
  if Params.Values[SFetchAll] <> '' then
    SQLConnection.SetOption(coFetchAll, LongInt(UpperCase(trim(Params.Values[SFetchAll])) = 'TRUE'));
  if Params.Values[SCharLength] <> '' then
    SQLConnection.SetOption(coCharLength, StrToInt(trim(Params.Values[SCharLength])));
  if Params.Values[SPrepared] <> '' then
    SQLConnection.SetOption(coPrepared, LongInt(UpperCase(trim(Params.Values[SPrepared])) = 'TRUE'));
  if Params.Values[SParamPrefix] <> '' then
    SQLConnection.SetOption(coParamPrefix, LongInt(UpperCase(trim(Params.Values[SParamPrefix])) = 'TRUE'));
  if Params.Values[STrimFixedChar] <> '' then
    SQLConnection.SetOption(coTrimFixedChar, LongInt(UpperCase(trim(Params.Values[STrimFixedChar])) = 'TRUE'));
  if Params.Values[SCharset] <> '' then
    SQLConnection.SetOption(coCharset, Integer(Params.Values[SCharset]));
  if Params.Values[SIntegerPrecision] <> '' then
    SQLConnection.SetOption(coIntegerPrecision, StrToInt(Params.Values[SIntegerPrecision]));
  if Params.Values[SSmallIntPrecision] <> '' then
    SQLConnection.SetOption(coSmallIntPrecision, StrToInt(Params.Values[SSmallIntPrecision]));
  if Params.Values[SFloatPrecision] <> '' then
    SQLConnection.SetOption(coFloatPrecision, StrToInt(Params.Values[SFloatPrecision]));
  if Params.Values[SBCDPrecision] <> '' then
    SQLConnection.SetOption(coBCDPrecision, Integer(Params.Values[SBCDPrecision]));
  if Params.Values[SEnableBoolean] <> '' then
    SQLConnection.SetOption(coEnableBoolean, Integer(Params.Values[SEnableBoolean]));
  {$ENDIF}
end;

function TrSQLConnection.GetVendorVersion: string;
begin
  {$ifdef mswindows}
  if VarIsNull(FVendorVersion) then
    FVendorVersion := GetFileVersion(GetModulePath(GetModuleHandle(PChar(ExtractFileName(VendorLib)))));
  {$endif}
  Result := VarToStr(FVendorVersion);
end;

{ TrSQLQuery }

constructor TrSQLQuery.Create(AOwner: TComponent);
begin
  inherited;
  {$IFDEF RSQLMONITOR}
  FDDE := CheckDDE;
  {$ENDIF}
  FMacroChar := DefaultMacroChar;
  FMacros := TParams.Create;
  FSQL := TStringList.Create;
  TStringList(SQL).OnChange := QueryChanged;
end;

procedure TrSQLQuery.CreateMacros(List: TParams; const Value: PChar);
begin
  CreateQueryParams(List, Value, True, MacroChar, ['.']);
end;

destructor TrSQLQuery.Destroy;
begin
  FSQL.Free;
  FMacros.Free;
  {$IFDEF RSQLMONITOR}
  if FDDE <> nil then FDDE.Free;
  {$ENDIF}
  inherited;
end;

procedure TrSQLQuery.DoBeforeOpen;
begin
  TSQLQuery(Self).SQL.Text := RealSQL;
  inherited;
end;

function TrSQLQuery.ExecSQL(ExecDirect: Boolean): Integer;
begin
  TSQLQuery(Self).SQL.Text := RealSQL;
  Result := inherited ExecSQL(ExecDirect);
end;

function TrSQLQuery.GetMacroCount: Word;
begin
  Result := FMacros.Count;
end;

function TrSQLQuery.GetMacros: TParams;
begin
  Result := FMacros;
end;

function TrSQLQuery.GetPrepared: boolean;
begin
  Result := inherited Prepared;   
end;

function TrSQLQuery.GetRealSQL: string;

  function ReplaceString(const S: string): string;
  var
    I, J, P, LiteralChars: Integer;
    Param: TParam;
    Found: Boolean;
  begin
    Result := S;
    for I := Macros.Count - 1 downto 0 do begin
      Param := Macros[I];
      if Param.DataType = ftUnknown then Continue;
      repeat
        P := Pos(MacroChar + Param.Name, Result);
        Found := (P > 0) and ((Length(Result) = P + Length(Param.Name)) or
          NameDelimiter(Result[P + Length(Param.Name) + 1], ['.']));
        if Found then begin
          LiteralChars := 0;
          for J := 1 to P - 1 do
            if IsLiteral(Result[J]) then Inc(LiteralChars);
          Found := LiteralChars mod 2 = 0;
          if Found then begin
            Result := Copy(Result, 1, P - 1) + Param.Text + Copy(Result,
              P + Length(Param.Name) + 1, MaxInt);
          end;
        end;
      until not Found;
    end;
  end;

var
  I: Integer;
begin
  Result := '';
  for I := 0 to SQL.Count - 1 do
    Result := Result + ReplaceString(SQL[I]) + #13#10;
  Result := copy( Result, 1, length(Result) - 2 );
end;

procedure TrSQLQuery.InternalOpen;
{$IFDEF RSQLMONITOR}
var S: string;
{$ENDIF}
begin
  {$IFDEF RSQLMONITOR}
  if FDDE <> nil then
    begin
      S := TextToSQLMonitor( Self, 'EXECUTE ' + SQLConnection.Name + ' ' + Name );
      FDDE.ExecuteMacro( PChar(S), false );
    end;
  {$ENDIF}
  inherited;
end;

function TrSQLQuery.MacroByName(const Value: string): TParam;
begin
  Result := FMacros.ParamByName(Value);
end;

procedure TrSQLQuery.PrepareStatement;
{$IFDEF RSQLMONITOR}
var S: string;
{$ENDIF}
begin
  {$IFDEF RSQLMONITOR}
  if FDDE <> nil then
    begin
      S := TextToSQLMonitor( Self, 'PREPARE ' + SQLConnection.Name + ' ' + Name );
      FDDE.ExecuteMacro( PChar(S), false );
    end;
  {$ENDIF}
  inherited;
end;

procedure TrSQLQuery.QueryChanged(Sender: TObject);
begin
  //TSQLQuery( Self ).SQL.Text := SQL.Text;
  RecreateMacros;
  TSQLQuery( Self ).SQL.Text := GetRealSQL;
end;

procedure TrSQLQuery.RecreateMacros;
var
  List: TParams;
begin
  List := TParams.Create;
  try
    CreateMacros(List, PChar(SQL.Text));
    List.AssignValues(FMacros);
    FMacros.Free;
    FMacros := List;
  except
    List.Free;
  end;
end;

procedure TrSQLQuery.SetMacroChar(Value: Char);
begin
  if Value <> FMacroChar then
    begin
      FMacroChar := Value;
      RecreateMacros;
    end;
end;

procedure TrSQLQuery.SetMacros(Value: TParams);
begin
  FMacros.AssignValues(Value);
end;

procedure TrSQLQuery.SetPrepared(Value: boolean);
begin
  if TSQLQuery(Self).SQL.Text <> RealSQL then TSQLQuery(Self).SQL.Text := RealSQL;
  inherited;
end;

procedure TrSQLQuery.SetSQL(Value: TStrings);
begin
  if SQL.Text <> Value.Text then
  begin
    Close;
    SQL.BeginUpdate;
    try
      SQL.Assign(Value);
    finally
      SQL.EndUpdate;
    end;
  end;
end;

{$IFDEF RSQLMONITOR}
type TSQLMonitorContainer = class
     private
       FDDE: TDDEClientConv;
       FSQLConnection: TSQLConnection;
       FSQLMonitor: TSQLMonitor;
       procedure SQLMonitorTrace( Sender: TObject; CBInfo: pSQLTRACEDesc;
                                  var LogTrace: Boolean);
     public
       constructor Create( ASQLConnection: TSQLConnection );
       destructor Destroy; override;
     end;

constructor TSQLMonitorContainer.Create( ASQLConnection: TSQLConnection );
begin
  FSQLConnection := ASQLConnection;
  FSQLMonitor := TSQLMonitor.Create( nil );
  FSQLMonitor.SQLConnection := ASQLConnection;
  FSQLMonitor.OnTrace := SQLMonitorTrace;
  FSQLMonitor.Active := true;
end;

destructor TSQLMonitorContainer.Destroy;
begin
  FSQLMonitor.Free;
  FDDE.Free;
  inherited;
end;

procedure TSQLMonitorContainer.SQLMonitorTrace( Sender: TObject; CBInfo: pSQLTRACEDesc;
                                  var LogTrace: Boolean);
var S, S2: string;

  // ,        
  (*
  function IsBad( const S: string ): boolean;
  var i: integer;
  begin
    Result := false;
    for i := Low( NoPassMessages ) to High( NoPassMessages ) do
      if NoPassMessages[i] = copy(S,1,length(NoPassMessages[i])) then
        begin
          Result := true;
          Break;
        end;
  end;
  *)

begin
  //      
  S2 := PChar( @CBInfo.pszTrace );
  //if IsBad( S2 ) then Exit;  
  S := '[SQLMONITOR ' + FSQLConnection.Name + sLineBreak;
  S2 := ReplaceStr( S2, sLineBreak, cDelimiter );
  S := S + S2;
  FDDE.ExecuteMacro( PChar(S), false );
end;

var Containers: array of TSQLMonitorContainer;

procedure FreeContainers;
var i: integer;
begin
  for i := 0 to High( Containers ) do Containers[i].Free;
  SetLength( Containers, 0 );
end;

//   SQL Connection  
procedure PrepareToMonitoring( SQLConnection: TSQLConnection );
var i: integer;
    DDE: TDDEClientConv;
begin
  DDE := CheckDDE;
  if DDE = nil then Exit;
  //  ,     SQLConnection
  for i := 0 to High( Containers ) do
    if Containers[i].FSQLConnection = SQLConnection then
      Exit;
  SetLength( Containers, High( Containers ) + 2 );
  Containers[ High( Containers ) ] := TSQLMonitorContainer.Create( SQLConnection );
  Containers[ High( Containers ) ].FDDE := DDE;  
end;

function GetParamText( Param: TParam ): string;
var S: string;
begin
  Result := Param.Name + cDelimiter + FieldTypeNames[ Param.DataType ];
  case Param.ParamType of
    ptInput: S := ' (Input)';
    ptOutput: S := ' (Output)';
    ptInputOutput: S := ' (InputOutput)';
    ptResult: S := ' (Result)';
  else
    S := '';
  end;
  Result := Result + S + cDelimiter;
  if Param.IsNull then S := '<NULL>' else
  if VarIsEmpty( Param.Value ) then S := '<Unassigned>' else
    S := Param.Text;
  Result := Result + S;
end;

//  ,     SQL - 
function TextToSQLMonitor( Q: TrSQLQuery; const Prefix: string ): string;
var i: integer;
begin
  Result := '[RSQLQUERY ' + Prefix + ']' + sLineBreak;
  Result := Result + ReplaceStr( Q.RealSQL, sLineBreak, cDelimiter );
  for i := 0 to Q.Params.Count - 1 do
    Result := Result + sLineBreak + GetParamText( Q.Params[i] );
end;

//  ,     SQL - 
function TextToSQLMonitor( Q: TrSQLStoredProc; const Prefix: string ): string; overload;
var i: integer;
begin
  Result := '[RSQLSTOREDPROC ' + Prefix + ']' + sLineBreak;
  Result := Result + ReplaceStr( 'StoredProc ' + Q.StoredProcName, sLineBreak, cDelimiter );
  for i := 0 to Q.Params.Count - 1 do
    Result := Result + sLineBreak + GetParamText( Q.Params[i] );
end;

function CheckDDE: TDDEClientConv;
begin
  Result := TDDEClientConv.Create( nil );
  try
    Result.ConnectMode := ddeAutomatic;
    if not Result.SetLink('RSQLMONITOR','DDE_RSQLMONITOR_SERVER') then
      try
        Result.Free;
      finally
        Result := nil;
      end;
  except
    try
      Result.Free;
    finally
      Result := nil;
    end;
  end;
end;

procedure FreeMonitoring;
begin
  FreeContainers;
end;
{$ENDIF}

{ TrSQLStoredProc }

constructor TrSQLStoredProc.Create(AOwner: TComponent);
begin
  inherited;
  {$IFDEF RSQLMONITOR}
  FDDE := CheckDDE;
  {$ENDIF}
end;

destructor TrSQLStoredProc.Destroy;
begin
  {$IFDEF RSQLMONITOR}
  if FDDE <> nil then FDDE.Free;
  {$ENDIF}
  inherited;
end;

function TrSQLStoredProc.ExecProc: Integer;
{$IFDEF RSQLMONITOR}
var S: string;
{$ENDIF}
var
  p: Boolean;
  vs: string;
  vi: Integer;
  vb: LongBool;
begin
  {$IFDEF RSQLMONITOR}
  if FDDE <> nil then
    begin
      S := TextToSQLMonitor( Self, 'EXECUTE ' + SQLConnection.Name + ' ' + Name );
      FDDE.ExecuteMacro( PChar(S), false );
    end;
  {$ENDIF}

  vb := SQLConnection.DriverName = 'Interbase';
  if vb then begin
    if SQLConnection is TrSQLConnection then
      vs := TrSQLConnection(SQLConnection).VendorVersion
    else vs := '';
    vb := vs <> '';
    if vb then begin
      Val(ExtractWord(1, vs, ['.']), vi, Integer(vb));
      if vb then vb := False
      else vb := vi < 6;
    end;
  end;

  p := Prepared;
  try
    Result := inherited ExecProc;
  finally
    if vb and not p then Prepared := False;
  end;
end;

procedure TrSQLStoredProc.PrepareStatement;
{$IFDEF RSQLMONITOR}
var S: string;
{$ENDIF}
begin
  {$IFDEF RSQLMONITOR}
  if FDDE <> nil then
    begin
      S := TextToSQLMonitor( Self, 'PREPARE ' + SQLConnection.Name + ' ' + Name );
      FDDE.ExecuteMacro( PChar(S), false );
    end;
  {$ENDIF}
  inherited;
end;

initialization

finalization
  {$IFDEF RSQLMONITOR}
  FreeContainers;
  {$ENDIF}

end.

