(*====================================================================
    File rTimeEdit.pas

    Components library for Delphi and BCB++.
    tTimeEdit component.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rTimeEdit.pas,v 1.5 2008/09/19 12:27:51 savchenko Exp $
======================================================================*)

unit rTimeEdit;

{$I POLARIS.INC}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, StdCtrls, Mask, RxSpin
  {$IFDEF POLARIS_D6}, Variants, MaskUtils{$ENDIF};

const WM_REDRAW = WM_USER + 400;
const WM_GOSEL = WM_USER + 401;

type
  TrTimeEdit = class(TCustomMaskEdit)
  private
    FWithSeconds: boolean;
    FButton: TRxSpinButton;
    procedure SetButtonStyle ( Value: TrSpinButtonStyle );
    function GetButtonStyle: TrSpinButtonStyle;
    procedure SetWithSeconds( Value: boolean );
    //    
    procedure ValidateText;
    procedure SetD( i: integer; Ch: char );
    function GetTime: Variant;
    procedure SetTime( Value: Variant );
    procedure TopClick( Sender: TObject );
    procedure BottomClick( Sender: TObject );
    //   (,   )  dX
    procedure ChangeSection( i: integer; dX: integer; MaxValue: integer );
    procedure WMRedraw( var Message: TMessage ); message WM_REDRAW;
    procedure WMGOSEL( var Message: TMessage ); message WM_GOSEL;
    procedure WMSetCursor(var Message: TWMSetCursor); message WM_SETCURSOR;
  protected
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress( var Key: char ); override;
    procedure DoExit; override;
    procedure DoEnter; override;
  public
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
    procedure Clear; {$IFDEF POLARIS_D5}override;{$ENDIF}
    procedure Change; override;
    property Time: Variant read GetTime write SetTime;    
  published
    property Anchors;
    property AutoSelect;
    property AutoSize;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelKind;
    property BevelWidth;
    property BiDiMode;
    property BorderStyle;
    property ButtonStyle: TrSpinButtonStyle read GetButtonStyle write SetButtonStyle;
    property CharCase;
    property Color;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property WithSeconds: boolean read FWithSeconds write SetWithSeconds;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

implementation

Const ShortTimeMask = '99:99;1; ';
      LongTimeMask = '99:99:99;1; ';

{ TrTimeEdit }

procedure TrTimeEdit.BottomClick(Sender: TObject);
begin
  Case SelStart of
    0,1: ChangeSection( 1, -1, 23 );
    3,4: ChangeSection( 4, -1, 59 );
    6,7: ChangeSection( 7, -1, 59 );
  end;
end;

procedure TrTimeEdit.Change;
begin
  inherited;
  if FButton <> Nil then FButton.Refresh;
end;

procedure TrTimeEdit.ChangeSection(i, dX: integer; MaxValue: integer);
Var n: integer;
    S: String;
begin
  n := StrToIntDef( copy( Text, i, 2 ), 0 );
  n := n + dX;
  if n < 0 then Exit;
  if n > MaxValue then Exit;
  S := IntToStr( n );
  if length(S) < 2 then S := '0' + S;
  SetD( i, S[1] );
  SetD( i + 1, S[2] );
end;

procedure TrTimeEdit.Clear;
begin
  if WithSeconds then Text := '  :  :  ' else Text := '  :  ';
end;

constructor TrTimeEdit.Create(AOwner: TComponent);
begin
  inherited;
  Text := '';
  EditMask := LongTimeMask;
  FWithSeconds := true;
  FButton := TRxSpinButton.Create( Self );
  FButton.Width := 15;
  FButton.Align := alRight;
  FButton.Parent := Self;
  FButton.OnBottomClick := BottomClick;
  FButton.OnTopClick := TopClick;
end;

destructor TrTimeEdit.Destroy;
begin
  FButton.Free;
  inherited;
end;

procedure TrTimeEdit.DoEnter;
begin
  inherited;
  PostMessage( Handle, WM_REDRAW, 0, 0 );
end;

procedure TrTimeEdit.DoExit;
begin
  inherited;
  ValidateText;
  if FButton <> Nil then FButton.Refresh;
end;

function TrTimeEdit.GetButtonStyle: TrSpinButtonStyle;
begin
  Result := sbsDefault;
  if FButton <> Nil then Result := FButton.ButtonStyle;
end;



function TrTimeEdit.GetTime: Variant;

  function SW( var S: String ): Word;
  begin
    Result := 0; S := trim( S ); if S = '' then Exit;
    Result := StrToIntDef( S, 0 );
  end;

Var S1, S2, S3: string;
    Hour, Min, Sec, MSec: Word;
begin
  // ,   - 
  if (WithSeconds and (Text = '  :  :  ')) or
     (not WithSeconds and (Text = '  :  ')) then
    begin
      Result := Null;
      Exit;
    end;
  //  
  S1 := copy( Text, 1, 2 );
  S2 := copy( Text, 4, 2 );
  if WithSeconds then S3 := copy( Text, 7, 2 ) else S3 := '0';
  //    
  Hour := SW( S1 );
  Min := SW( S2 );
  Sec := SW( S3 );
  MSec := 0;
  // -----
  Result := EncodeTime( Hour, Min, Sec, MSec );  
end;

procedure TrTimeEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if Key = VK_UP then TopClick( Self ) else
  if Key = VK_DOWN then BottomClick( Self );
  inherited;
end;

procedure TrTimeEdit.KeyPress(var Key: char);
begin
  //  
  Case SelStart of
    0:   if not (Key in ['0'..'2']) then Key := #0;
    1:   if (Text[1] = '2') and not (Key in ['0'..'3']) then Key := #0;
    3,6: if not (Key in ['0'..'5']) then Key := #0;
  end;
  //  
  //  1-  ,    2-
  if (SelStart = 1) and (Text[1] = ' ') then SetD( 1, '0' );
  //  2-  ,   - ...
  if (Text[1] <> ' ') and (Text[2] = ' ') and (SelStart >= 3) then
    begin
      SetD( 2, Text[1] );
      SetD( 1, '0' );
    end;
  //   ,      ,    
  if (Text[1] = ' ') and (Text[2] = ' ') and (SelStart >= 3) then
    begin
      SetD( 1, '0' );
      SetD( 2, '0' );
    end;
  ///  1-   ,     2-,   1-  '0'
  if (Text[4] = ' ') and (SelStart = 4) then SetD( 4, '0' );
  //  1-    ,  2- ,      
  if (Text[4] <> ' ') and (Text[5] = ' ') and (SelStart in [0,1,6,7]) then
    begin
      SetD( 5, Text[4] );
      SetD( 4, '0' );
    end;
  //   ,    ,     
  if (Text[4] = ' ') and (Text[5] = ' ') and (SelStart in [6,7]) then
    begin
      SetD( 4, '0' );
      SetD( 5, '0' );
    end;
  //  1-   ,    2-,   1-  0
  if (Text[7] = ' ') and (SelStart = 7) then SetD( 7, '0' );
  //  2-   ,  1- ,     
  if (Text[8] = ' ') and (Text[7] <> ' ') and (SelStart in [0,1,3,4]) then
    begin
      SetD( 8, Text[7] );
      SetD( 7, '0' );
    end;
  // -----
  inherited;
end;

procedure TrTimeEdit.SetButtonStyle(Value: TrSpinButtonStyle);
begin
  if FButton = Nil then Exit;
  FButton.ButtonStyle := Value;
end;

procedure TrTimeEdit.SetD(i: integer; Ch: char);
Var LastSelStart: integer;
    LastSelLength: integer;
begin
    LastSelStart := SelStart;
    LastSelLength := SelLength;
    try
      Text := copy( Text, 1, i-1) + Ch + copy( Text, i+1, length(Text)-i);
    finally
      SelStart := LastSelStart;
      SelLength := LastSelLength;
    end;
end;

procedure TrTimeEdit.SetTime(Value: Variant);
Var Hour, Min, Sec, MSec: Word;
    S: String;

  //  Word -   
  function GetDigit( i: Word ): string;
  begin
    Result := IntToStr( i );
    if length( Result ) < 2 then Result := '0' + Result;
  end; 

begin
  //   
  if VarIsNull( Value ) then
    begin
      Clear;
      Exit;
    end;
  //  ,     
  DecodeTime( Value, Hour, Min, Sec, MSec );
  S := GetDigit( Hour ) + ':' + GetDigit( Min );
  if WithSeconds then S := S + ':' + GetDigit( Sec );
  Text := S;    
end;

procedure TrTimeEdit.SetWithSeconds(Value: boolean);
begin
  FWithSeconds := Value;
  if Value then EditMask := LongTimeMask else EditMask := ShortTimeMask;
end;

procedure TrTimeEdit.TopClick(Sender: TObject);
begin
  Case SelStart of
    0,1: ChangeSection( 1, 1, 23 );
    3,4: ChangeSection( 4, 1, 59 );
    6,7: ChangeSection( 7, 1, 59 );
  end;
end;

procedure TrTimeEdit.ValidateText;
Var i: integer;
begin
  //     
  if WithSeconds and (Text = '  :  :  ') then Exit;
  if not WithSeconds and (Text = '  :  ') then Exit;
  //  1    
  if (Text[1] <> ' ') and (Text[2] = ' ') then
    begin
      SetD( 2, Text[1] );
      SetD( 1, '0' );
    end;
  if (Text[4] <> ' ') and (Text[5] = ' ') then
    begin
      SetD( 5, Text[4] );
      SetD( 4, '0' );
    end;
  if (Text[7] <> ' ') and (Text[8] = ' ') then
    begin
      SetD( 8, Text[7] );
      SetD( 7, '0' );
    end;
  //       0
  for i := 1 to length( Text ) do if Text[i] = ' ' then SetD( i, '0' );
  // ---
end;

procedure TrTimeEdit.WMGOSEL(var Message: TMessage);
begin
  SelStart := Message.WParam;
  SelLength := Message.LParam;
end;

procedure TrTimeEdit.WMRedraw(var Message: TMessage);
begin
  if FButton <> Nil then FButton.Refresh;
end;

procedure TrTimeEdit.WMSetCursor(var Message: TWMSetCursor);
var
  P: TPoint;
begin
  GetCursorPos(P);
  with ClientRect do
    if PtInRect(Bounds(Right - FButton.Width, Top, FButton.Width, Bottom - Top),
      ScreenToClient(P)) then
{$IFDEF WIN32}
      Windows.SetCursor(LoadCursor(0, IDC_ARROW))
{$ELSE}
      WinProcs.SetCursor(LoadCursor(0, IDC_ARROW))
{$ENDIF}
    else inherited;
end;

end.
