(*====================================================================
    File rUtils.pas

    Components library for Delphi, Kylix and BCB++.
    Procedures and functions.

    Copyright  1998-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rUtils.pas,v 1.19 2008/10/06 11:58:10 khmelevskiy Exp $
======================================================================*)
unit rUtils;

{$I POLARIS.INC}

interface

uses
{$IFDEF LINUX}
  Types, Libc, rLinux,
{$ELSE}
  Windows, Registry,
{$ENDIF}
  Classes, SysUtils, TypInfo, MaxMin, rConst, rRegIni,
  {$IFDEF POLARIS_D6}RxStrUtils, Variants, RTLConsts{$ELSE}StrUtils{$ENDIF};

type
  TrDate = record
             D,M,Y: Word;
           end;

  TBitmapResKind =
    (rkNone, rkBitmap, rkIcon, rkCursor, rkEMF, rkWMF
     {$IFDEF GIF}, rkGIF{$ENDIF}{$IFDEF JPEG}, rkJPEG{$ENDIF});

  //      (0- 1; 1- 2..4; 2- 0,5..9)
  // Currency names for numerals (0- 1; 1- 2..4; 2- 0,5..9)
  TCurrencyNames = array [0..2] of string;

  //  (, ., ., . )
  TGenders = (gNeuter,gMale,gFemale,gPlural);
  TGenderNames = array[TGenders] of string;

  //   ( ,   ,    )
  // File locations
  TrFileLocation = (flDefault, flProgram, flModule);

const
{$IFNDEF POLARIS_D4}
  TIME_ZONE_ID_STANDARD = 1;
  TIME_ZONE_ID_DAYLIGHT = 2;
{$ENDIF}

  NO = False;            YES = True;
  CRLF       = #13#10;
{$IFNDEF POLARIS_D6}
  sLineBreak = CRLF;
{$ENDIF}
  crYes      = '1';
  crNo       = '0';
  crOK       = 'OK';
  crEmptyStr = '';
  crBlank    = ' ';
  crKeyUp    = '..';
  crEllipsis = '...';
  crKeyUp2   = crKeyUp + '\' + crKeyUp;
  crExact: array[Boolean] of string = (crNo, crYes);

  WIN_CAP_DELIM = ' - ';

  NewStrings: TGenderNames = (srNewN,srNewM,srNewF,srNewP);

  Rubles:  TCurrencyNames = ('','','');
  Copecks: TCurrencyNames = ('','','');

  Dollars: TCurrencyNames = ('dollar','dollars','dollars');
  Cents:   TCurrencyNames = ('cent','cents','cents');

  DateSeparatorFix: Char = '.';
  TimeSeparatorFix: Char = ':';
  ShortDateFormatFix = 'dd.mm.yyyy';
  ShortTimeFormatFix = 'hh:nn:ss';
  DecimalSeparatorFix: Char  = '.';

{$IFNDEF POLARIS_D6}
  PathDelim  = '\';
  DriveDelim = ':';
  PathSep    = ';';
  sLineBreak = CRLF;
{$ENDIF}
  crVersionSep = '.';

  RegStringDelim = #4;
  RegStringSoftDelim = ';';

  DefBRK = rkBitmap;
{$IFDEF LINUX}
  RT_BITMAP     = 'BMP';
  RT_GROUP_ICON = 'ICO';
  RT_CURSOR     = 'CUR';
{$ENDIF}
  ResKindNames: array[TBitmapResKind] of PChar =
    (crEmptyStr,RT_BITMAP,RT_GROUP_ICON,RT_CURSOR,'EMF','WMF'
     {$IFDEF GIF},'GIF'{RT_GIF}{$ENDIF}{$IFDEF JPEG},'JPEG'{$ENDIF});

  crAnyFileMask = {$ifdef linux} '*' {$else} '*.*' {$endif};

var
  RHandles: TList;
{$ifdef mswindows}
  use_wine: Boolean;
{$endif}

//========   dates

//        . 
// ONLY RUSSIAN !!!
function AdaptedDateStr(Date: TDateTime): string;

//    
// Days number in month of Date
function DaysOfMonth(Date: TDateTime): Integer;

//     
// Choice maximum date from list  (use MaxDateTime from MaxMin.pas)
function MaxDate(const D: array of TDateTime): TDateTime; {$IFDEF POLARIS_D6}deprecated;{$ENDIF}

//     
// Choice minimum date from list  (use MinDateTime from MaxMin.pas)
function MinDate(const D: array of TDateTime): TDateTime; {$IFDEF POLARIS_D6}deprecated;{$ENDIF}

//  
// Day of week
function rDayOfWeek(dDate: TDateTime): Integer;

//========   strings

//      
// Delete and insert symbols in string
function Stuff(S: string; Index,Count: Integer; Source: string): string;

//  \n, \r, \t  \NNN  
// Convert \n, \r, \t and \NNN constructions into chars
function ConvertCodes(Source: string): string;

//           
// Convert symbols from one symbols set to other symbols set
function ConvertSymbols(S, Source, Target: string): string;

//  ANSI-    Unicode- 
// Convert string to Unicode string
// Example:  'ABC'  ->  '#65#66#67#1040#1041#1042'
function ConvertStrToUnicode(const Source: string): string;

//    Unicode-   ANSI-
// Convert Unicodes in Source to symbols
// Example:  '#1040#1041#1042ABC'  ->  'ABC'
function ConvertUnicodeToStr(const Source: string): string;

//     S
// Number of delimiters in string S
function GetDelimCount(S: String; Delims: TCharSet): Integer;

// N     S
// Returns a substring of a string S (N last symbols)
function RightCopy(S: String; N: Integer): String;

//     S 
// Position of character in string S from right
function PosR(C: Char; S: String): Integer;

//       S
// Position of quoted with delimiters substring in string S
function PosWordDelims(Srch, S: String; Delims: TCharSet): Integer;

//     - c 
// Add first word into string-list with delimiters
function AddFirstToList(List, S: String; Delim: String): String;

//  TList    
// Convert TList to string with delimiters
function ListToStr(List: TList; Delim: String): String;

//            
//  
// Convert dates and numbers in strings and vice versa with fix delimiters
function StrToDateFix(const S: string): TDateTime;
function StrToTimeFix(const S: string): TDateTime;
function StrToDateTimeFix(const S: string): TDateTime;
function DateToStrFix(Date: TDateTime): string;
function TimeToStrFix(Time: TDateTime): string;
function DateTimeToStrFix(DateTime: TDateTime): string;
function StrToFloatFix(const S: string): Extended;
{$IFDEF POLARIS_D6}
function StrToFloatDefFix(const S: string; const Default: Extended): Extended;
{$ENDIF}
function FloatToStrFix(Value: Extended): string;
function FormatFloatFix(const Format: string; Value: Extended): string;

//     
// Initials formatting
function FormatFIO(S: string): string;

//     
// Replace substring after delimiter
function ChangeEndStr(Dest, Delim, Source: string): string;

//  \  /   
// Concatenates \ or / to Path
function AddSlash(Path: string): string;

//       
function IsFullPath(AFileName: String): Boolean;

{$IFNDEF POLARIS_D6}
//    
function DirectoryExists(const Name: string): Boolean;
{$ENDIF}

//     
// String truncation to Len simbols
function StrTrunc(Source: string; Len: Integer; TermStr: string): string;

//      TCurrencyNames
// Check number to including in TCurrencyNames
function TestNum(Num: Integer): Integer;

//       
// Convert integer part of Sum in words
function NumInWords(Sum: Extended; Gender: TGenders {$IFDEF POLARIS_D4}= gMale{$ENDIF}): string;

//   
// Convert currency sum in words
function MoneyInWords(Sum: Extended; MainCur, SecCur: TCurrencyNames; MainGender: TGenders;
                      Decimals: Integer {$IFDEF POLARIS_D4}= 2{$ENDIF}): string;

//    "N -"
// Format string "N something"
function NumSomething(Num: Integer; Something: String; IncludeNum: Boolean {$IFDEF POLARIS_D4}= True{$ENDIF}): String;
{$IFDEF POLARIS_D4}overload;
function NumSomething(Num: Integer; Something: TCurrencyNames;
  IncludeNum: Boolean = True): String; overload;
{$ENDIF}

//========   numbers

//     
// Cyclic increment inside range
function IncCycle(var Value: Integer;const Offset, MinValue, MaxValue: Integer): Integer;

//  
// Number sign
function Sign(Value: Extended): Integer;

//   
// Raising to integer power
function RaisePower(Value,Power: Integer): Integer;

//    Extended
// Raising to power
function fPower(Value, Power: Extended): Extended;

// 
// Extended round
function fRound(X: Extended; Precision: Integer): Extended;

// ,    0,    
// Round, but return X if round result is 0
function SoftRound(X: Extended; Precision: Integer): Extended;

//   (  =0,  DefValue)
// Safe division (if by zero then DefValue returns)
function DivZero(X, Y: Extended; DefValue: Extended {$IFDEF POLARIS_D4}=0{$ENDIF}): Extended;

//========   OS

//   
// Get OS user name
function GetOSUser: string;

//  
// Get computer name
function GetCompName: string;

//      
// Get module fullpath & name
function GetModulePath(hModule: THandle): string; {$ifdef linux} overload; {$endif}
{$ifdef linux}
function GetModulePath(hModule: Pointer): string; overload;
{$endif}

// config files directories
function GetGlobalConfigDir(): string;
function GetLocalConfigDir(): string;

{$ifdef mswindows}

//   
// Set system time
procedure SetOSTime(D: TDateTime);

//     (Flag = EWX_xxxx, . ExitWindowsEx)
// Logs off, shuts down, or shuts down and restarts the system
procedure ExitOS(Flag: UINT);

{$endif mswindows}

//  Version Info
// Load VersionInfo
function GetVersionInfo(VerList: TStrings; FileName: string {$IFDEF POLARIS_D4}= crEmptyStr{$ENDIF}): Boolean;

//  
// File version
function GetFileVersion(FileName: string {$IFDEF POLARIS_D4}= crEmptyStr{$ENDIF}): string;

{$ifdef mswindows}

//========   registry

//  
// Read value

function RegReadValue(Reg: TRegistry; Name: string; Default: string {$IFDEF POLARIS_D4}= crEmptyStr{$ENDIF}): string;
{$IFDEF POLARIS_D4}overload;
function RegReadValue(Reg: TRegistry; Name: string; Default: Integer = 0): Integer; overload;
function RegReadValue(Reg: TRegistry; Name: string; Default: Extended = 0): Extended; overload;
function RegReadValue(Reg: TRegistry; Name: string; Default: Boolean = False): Boolean; overload;
function RegReadValue(Reg: TRegistry; Name: string; Default: TDateTime = 0): TDateTime; overload;
{$ENDIF} // POLARIS_D4
{$endif mswindows}

{$IFDEF POLARIS_D4}
function RegReadValue(Reg: TrRegIni; Name: string; Default: string = crEmptyStr): string; overload;
function RegReadValue(Reg: TrRegIni; Name: string; Default: Integer = 0): Integer; overload;
function RegReadValue(Reg: TrRegIni; Name: string; Default: Extended = 0): Extended; overload;
function RegReadValue(Reg: TrRegIni; Name: string; Default: Boolean = False): Boolean; overload;
function RegReadValue(Reg: TrRegIni; Name: string; Default: TDateTime = 0): TDateTime; overload;
{$ENDIF} // POLARIS_D4

//   Source  RootKey  RegKey
// Separate string Source to RootKey and RegKey
function SeparateRegKey(Source: string; var TargetRoot: HKEY; var TargetKey: string): Boolean;

function RegKeyToKeyName(aRegKey: HKEY): string;

//    XX.XX.XX.XX     
// Separates version represented in XX.XX.XX.XX format into most and least significant 32 bits
function SeparateVersion(Version: string; var MS, LS: DWORD): Boolean;

//    
// Compare two file versions
function CompareVersion(Ver1, Ver2: string): Integer;

//========    other

//   Variant'
// Compare two Variants
function CompareVar(Var1, Var2: Variant): Integer;

//   TList  Source  Dest
// Copy TList items from Source to Dest
function AssignList(Dest, Source: TList): Integer;

//   
// Copy object properties
procedure CopyObject(Source, Dest: TObject);

//    
// Choice from two variants or strings
function IIF(Switch: Boolean; Var1,Var2: Variant): Variant;
{$IFDEF POLARIS_D4}overload;
function IIF(Switch: Boolean; Var1,Var2: string): string; overload;
{$ENDIF}

//         
// Return properly index of S in Items with match case
function GetRealIndex(Items: TStrings; FirstIndex: Integer; S: string): Integer;

//      (Item1, Item2,...)  Items  Values
// Return values list of numbered key from Items into Values
function ReadEnumValues(Items: TStrings; Values: TStrings; ItemName, ItemCountName: string;
                        FirstIndex: Integer {$IFDEF POLARIS_D4}= 1{$ENDIF}): Integer;

//      
// Checks inheritance from one of classes
function InheritsFromOneOf(const aClass: TClass; aClasses: array of TClass): Boolean; overload;
function InheritsFromOneOf(const aObject: TObject; aClasses: array of TClass): Boolean; overload;

{$ifdef mswindows}
procedure SaveToReg(Reg: TRegistry; RegKey, FKey: string; Value: Integer); {$IFDEF POLARIS_D4}overload;
procedure SaveToReg(Reg: TRegistry; RegKey, FKey: string; Value: string); overload; {$ENDIF}
function LoadFromReg(Reg: TRegistry; RegKey, FKey: string; Default: string {$IFDEF POLARIS_D4}= crEmptyStr{$ENDIF}): string;
{$endif mswindows}

implementation

function AdaptedDateStr(Date: TDateTime): string;
var
  DMY: TrDate;
  newFmt,strMon: string;
begin
  with DMY do begin
    {     }
    DecodeDate(Date,Y,M,D);
    strMon := AnsiLowerCase(LongMonthNames[M]);
    {$ifdef mswindows}
    if strMon[Length(strMon)] in ['',''] then
      strMon := Stuff(strMon,Length(strMon),1,'')
    else strMon := strMon+'';
    {$endif}
    {   }
    newFmt := LongDateFormat;
    repeat
      D := Pos('MMMM',UpperCase(newFmt));
      if D = 0 then Break;
      newFmt := Stuff(newFmt,D,4,'"'+strMon+'"');
    until D=0;
  end;
  Result := FormatDateTime(newFmt,Date)
end;

function DaysOfMonth(Date: TDateTime): Integer;
var dmy: TrDate;
begin
  with dmy do begin
    DecodeDate(Date,y,m,d);
    if m in [4,6,9,11] then Result := 30 else Result := 31;
    if m = 2 then
      if IsLeapYear(y) then Result := 29 else Result := 28;
  end;
end;

function MaxDate(const D: array of TDateTime): TDateTime;
{var
  n, i: Integer;}
begin
  Result := MaxDateTime(D);
{  n := High(D);
  if n-Low(D) > 0 then begin
    Result := D[Low(D)];
    I := Low(D);
    repeat
      if D[I] > Result
      then Result := D[I];
      Inc(I);
    until I > n;
  end else Result := 0;}
end;

function MinDate(const D: array of TDateTime): TDateTime;
{var
  n, i: Integer;}
begin
  Result := MinDateTime(D);
{  n := High(D);
  if n-Low(D) > 0 then begin
    Result := D[Low(D)];
    I := Low(D);
    repeat
      if D[I] < Result
      then Result := D[I];
      Inc(I);
    until I > n;
  end else Result := 0;}
end;

function rDayOfWeek(dDate: TDateTime): Integer;
begin
  Result := DayOfWeek(dDate);
{$IFDEF WEEK_MONDAY}
  IncCycle(Result, -1, 1, 7);
{$ENDIF}
end;

function Stuff(S: string; Index,Count: Integer; Source: string): string;
begin
  Result := S;
  Delete(Result,Index,Count);
  Insert(Source,Result,Index);
end;

function ConvertCodes(Source: string): string;
var
  i, j, l: Integer;
begin
  i := 0;
  l := Length(Source);
  Result := crEmptyStr;
  while i < l do begin
    j := Pos('\', Copy(Source, i + 1, l)) + i;
    if j = i then begin
      Result := Result + Copy(Source, i + 1, l);
      i := l;
    end else begin
      Result := Result + Copy(Source, i + 1, j - i - 1);
      case Source[j + 1] of
        '\': begin
               Result := Result + '\';
               i := j + 1;
             end;
        'n': begin
               Result := Result + #10; //sLineBreak;
               i := j + 1;
             end;
        'r': begin
               Result := Result + #13;
               i := j + 1;
             end;
        't': begin
               Result := Result + #9;
               i := j + 1;
             end;
        '0'..'9': begin
                    i := j;
                    while (j < l) and (Source[j + 1] in ['0'..'9']) do begin
                      if StrToInt(Copy(Source, i + 1, j - i + 1)) > 255 then Break;
                      Inc(j);
                    end;
                    Result := Result + Chr(StrToInt(Copy(Source, i + 1, j - i)));
                    i := j;
                  end
        else begin
          i := j;
        end;
      end;
    end;
  end;
end;

function ConvertSymbols(S, Source, Target: string): string;
var
  i, j: Integer;
begin
  Result := crEmptyStr;
  for i:=1 to Length(S) do begin
    j := Pos(S[i], Source);
    if (j <> 0) and (Length(Target) >= j) then
      Result := Result + Target[j]
    else Result := Result + S[i];
  end
end;

function ConvertStrToUnicode(const Source: string): string;
var
  i, l: Integer;
  p: Pointer;
  pw: PWord;
begin
  Result := crEmptyStr;
  l := 2 * Length(Source) + 1;
  GetMem(p, l);
  try
    StringToWideChar(Source, p, l);
    pw := p;
    for i := 1 to Length(Source) do begin
      Result := Result + '#' + IntToStr(pw^);
      Inc(pw);
    end;
  finally
    FreeMem(p);
  end;
end;

function ConvertUnicodeToStr(const Source: string): string;
var
  i, j, l: Integer;
  w: record
    ch: Word;
    nl: Byte;
  end;
begin
  i := 0;
  l := Length(Source);
  Result := crEmptyStr;
  w.nl := 0;
  while i < l do begin
    j := Pos('#', Copy(Source, i + 1, l)) + i;
    if j = i then begin
      Result := Result + Copy(Source, i + 1, l);
      i := l;
    end else begin
      Result := Result + Copy(Source, i + 1, j - i - 1);
      case Source[j + 1] of
        '0'..'9': begin
                    i := j;
                    while (j < l) and (Source[j + 1] in ['0'..'9']) do begin
                      if StrToInt(Copy(Source, i + 1, j - i + 1)) > 65535 then Break;
                      Inc(j);
                    end;
                    w.ch := StrToInt(Copy(Source, i + 1, j - i));
                    Result := Result + WideCharToString(@w);
                    i := j;
                  end
        else begin
          Result := Result + '#';
          i := j;
        end;
      end;
    end;
  end;
end;

function GetDelimCount(S: String; Delims: TCharSet): Integer;
var
  I: Integer;
begin
  Result := 0;
  if Length(S) > 0 then
    for I := 1 to Length(S) do
      if S[I] in Delims then Inc(Result);
end;

function RightCopy(S: String; N: Integer): String;
var
  l: Integer;
begin
  l := Length(S);
  Result := Copy(S, l-N+1, l);
end;

function PosR(C: Char; S: String): Integer;
var
  I: Integer;
begin
  Result := 0;
  if Length(S) > 0 then
    for I := Length(S) downto 1 do
      if S[I] = C then begin
        Result := I;
        exit;
      end;
end;

function PosWordDelims(Srch, S: String; Delims: TCharSet): Integer;
var
  i, n, j: Integer;
begin
  Result := 0;
  n := Length(Srch);
  j := 1;
  repeat
    i := NPos(Srch, S, j);
    if i = 0 then Break;
    if ((i = 1) or (S[i-1] in Delims)) and
       ((i = Length(S)-n+1) or (S[i+n] in Delims)) then  begin
      Result := i;
      Break;
    end;
    inc(j);
  until False;
end;

function AddFirstToList(List, S: String; Delim: String): String;
begin
  if Trim(List) = crEmptyStr then Result := S
  else Result := S + Delim + List;
end;

function ListToStr(List: TList; Delim: String): String;
var
  j: Integer;
begin
  Result := crEmptyStr;
  for j := 0 to List.Count - 1 do begin
    Result := Result + IntToStr(Integer(List.Items[j]));
    if j < List.Count - 1 then Result := Result + Delim;
  end;
end;

function StrToDateFix(const S: string): TDateTime;
var
  oldSep1: Char;
  oldFormat1: string;
begin
  oldSep1 := DateSeparator;
  oldFormat1 := ShortDateFormat;
  DateSeparator := DateSeparatorFix;
  ShortDateFormat := ShortDateFormatFix;
  try
    Result := StrToDate(S);
  finally
    DateSeparator := oldSep1;
    ShortDateFormat := oldFormat1;
  end;
end;

function StrToTimeFix(const S: string): TDateTime;
var
  oldSep1: Char;
  oldFormat1: string;
begin
  oldSep1 := TimeSeparator;
  oldFormat1 := ShortTimeFormat;
  TimeSeparator := TimeSeparatorFix;
  ShortTimeFormat := ShortTimeFormatFix;
  try
    Result := StrToTime(S);
  finally
    TimeSeparator := oldSep1;
    ShortTimeFormat := oldFormat1;
  end;
end;

function StrToDateTimeFix(const S: string): TDateTime;
var
  oldSep1, oldSep2: Char;
  oldFormat1, oldFormat2: string;
begin
  oldSep1 := DateSeparator;
  oldFormat1 := ShortDateFormat;
  oldSep2 := TimeSeparator;
  oldFormat2 := ShortTimeFormat;
  DateSeparator := DateSeparatorFix;
  ShortDateFormat := ShortDateFormatFix;
  TimeSeparator := TimeSeparatorFix;
  ShortTimeFormat := ShortTimeFormatFix;
  try
    Result := StrToDateTime(S);
  finally
    DateSeparator := oldSep1;
    ShortDateFormat := oldFormat1;
    TimeSeparator := oldSep2;
    ShortTimeFormat := oldFormat2;
  end;
end;

function DateToStrFix(Date: TDateTime): string;
var
  oldSep1: Char;
  oldFormat1: string;
begin
  oldSep1 := DateSeparator;
  oldFormat1 := ShortDateFormat;
  DateSeparator := DateSeparatorFix;
  ShortDateFormat := ShortDateFormatFix;
  try
    Result := DateToStr(Date);
  finally
    DateSeparator := oldSep1;
    ShortDateFormat := oldFormat1;
  end;
end;

function TimeToStrFix(Time: TDateTime): string;
var
  oldSep1: Char;
  oldFormat1: string;
begin
  oldSep1 := TimeSeparator;
  oldFormat1 := ShortTimeFormat;
  TimeSeparator := TimeSeparatorFix;
  ShortTimeFormat := ShortTimeFormatFix;
  try
    Result := TimeToStr(Time);
  finally
    TimeSeparator := oldSep1;
    ShortTimeFormat := oldFormat1;
  end;
end;

function DateTimeToStrFix(DateTime: TDateTime): string;
var
  oldSep1, oldSep2: Char;
  oldFormat1, oldFormat2: string;
begin
  oldSep1 := DateSeparator;
  oldFormat1 := ShortDateFormat;
  oldSep2 := TimeSeparator;
  oldFormat2 := ShortTimeFormat;
  DateSeparator := DateSeparatorFix;
  ShortDateFormat := ShortDateFormatFix;
  TimeSeparator := TimeSeparatorFix;
  ShortTimeFormat := ShortTimeFormatFix;
  try
    Result := DateTimeToStr(DateTime);
  finally
    DateSeparator := oldSep1;
    ShortDateFormat := oldFormat1;
    TimeSeparator := oldSep2;
    ShortTimeFormat := oldFormat2;
  end;  
end;

function StrToFloatFix(const S: string): Extended;
begin
  Result := StrToFloat(ReplaceStr(S, DecimalSeparatorFix, DecimalSeparator));
end;

{$IFDEF POLARIS_D6}
function StrToFloatDefFix(const S: string; const Default: Extended): Extended;
begin
  Result := StrToFloatDef(ReplaceStr(S, DecimalSeparatorFix, DecimalSeparator), Default);
end;
{$ENDIF}

function FloatToStrFix(Value: Extended): string;
var
  oldSep1: Char;
begin
  oldSep1 := DecimalSeparator;
  DecimalSeparator := DecimalSeparatorFix;
  try
    Result := FloatToStr(Value);
  finally
    DecimalSeparator := oldSep1;
  end;
end;

function FormatFloatFix(const Format: string; Value: Extended): string;
var
  oldSep1: Char;
begin
  oldSep1 := DecimalSeparator;
  DecimalSeparator := DecimalSeparatorFix;
  try
    Result := FormatFloat(Format,Value);
  finally
    DecimalSeparator := oldSep1;
  end;
end;

function FormatFIO(S: string): string;
begin
  if S = crEmptyStr then exit;
  Result := ExtractWord(1, S, [crBlank]);
  if WordCount(S, [crBlank]) > 1 then
    Result := Result + crBlank + AnsiUpperCase(String(ExtractWord(2, S, [crBlank])[1]))+'.';
  if WordCount(S, [crBlank]) > 2 then
    Result := Result + crBlank + AnsiUpperCase(String(ExtractWord(3, S, [crBlank])[1]))+'.';
end;

function ChangeEndStr(Dest, Delim, Source: string): string;
var
  s: string;
begin
  if Pos(Delim, Dest) = 0 then s := Dest
  else s := Copy(Dest, 1, Pos(Delim, Dest)-1);
  if Source = crEmptyStr then Result := s
                 else Result := s+Delim+Source;
end;

function AddSlash(Path: string): string;
begin
  {$IFDEF MSWINDOWS}
  Result := Path;
  if (Path <> crEmptyStr) and (Pos(Copy(Path, Length(Path), 1), DriveDelim + PathDelim) = 0) then
    Result := Result + PathDelim;
  {$ELSE}
  Result := IncludeTrailingPathDelimiter(Path);
  {$ENDIF}
end;

function IsFullPath(AFileName: String): Boolean;
begin
  Result := (ExtractFileDrive(AFileName) <> crEmptyStr) or (Copy(AFileName, 1, 1) = PathDelim);
end;

{$IFNDEF POLARIS_D6}
function DirectoryExists(const Name: string): Boolean;
var
  Code: Integer;
begin
  Code := GetFileAttributes(PChar(Name));
  Result := (Code <> -1) and (FILE_ATTRIBUTE_DIRECTORY and Code <> 0);
end;
{$ENDIF}

function StrTrunc(Source: string; Len: Integer; TermStr: string): string;
begin
  if Len > Length(Source) then Result := Source
  else
    Result := Copy(Source,1,Len-Length(TermStr))+TermStr;
end;

function TestNum(Num: Integer): Integer;
begin
  if (Num mod 10 = 1) and (Num mod 100 <> 11) then Result := 0
  else if (Num mod 10 in [2..4]) and not(Num mod 100 in [12..14]) then Result := 1
  else Result := 2;
end;

function NumInWords(Sum: Extended; Gender: TGenders{$IFDEF POLARIS_D4} = gMale{$ENDIF}): string;
const
  a1: array [1..19] of string =
    (sr1,sr2,sr3,sr4,sr5,sr6,sr7,sr8,sr9,
     sr10,sr11,sr12,sr13,sr14,sr15,sr16,sr17,sr18,sr19);
  a10: array [2..9] of string =
    (sr20,sr30,sr40,sr50,sr60,sr70,sr80,sr90);
  a100: array [1..9] of string =
    (sr100,sr200,sr300,sr400,sr500,sr600,sr700,sr800,sr900);
  aExp: array [1..5] of record
                          Num: Integer;
                          str: string;
                        end =
    ((num:12; str:sr10_12;),
     (num:9;  str:sr10_9; ),
     (num:6;  str:sr10_6; ),
     (num:3;  str:sr10_3; ),
     (num:0;  str:crEmptyStr;     ));
var
  pNum, Divider: Extended;
  i, nGender, mNum: LongInt;
begin
  Result := crEmptyStr;
  if Sum < 0 then Result := '- ';
  pNum := Abs(Trunc(Sum));
  for i:=1 to 5 do begin
    Divider := Round(fPower(10,aExp[i].Num));
    mNum := Trunc(pNum/Divider);
    if mNum > 0 then begin
      if mNum > 99 then begin
        Result := Result+a100[mNum div 100] + crBlank;
        mNum := mNum mod 100;
      end;
      if mNum > 19 then begin
        Result := Result+a10[mNum div 10] + crBlank;
        mNum := mNum mod 10;
      end;
      if mNum > 0 then begin
        if i = 5 then
          nGender := Ord(Gender)+1
        else
          nGender := Pos(ExtractWord(1, aExp[i].str, [',']), srGenders);
        if (nGender = 0) then nGender := 1;
        nGender := Min(nGender, WordCount(a1[mNum], [',']));
        Result := Result+ExtractWord(nGender, a1[mNum], [',']) + crBlank;
      end;
      if (i <> 5) and (Trunc(pNum/Divider)<>0) then
        // name of numeral
        Result := Result+
                  ExtractWord( Min(TestNum(mNum)+2, WordCount(aExp[i].str, [','])),
                               aExp[i].str, [',']) + crBlank;
    end;
    pNum := Round(Frac(pNum/Divider)*Divider);
  end;
  if Result = crEmptyStr then Result := AnsiUpperCase(sr0) + crBlank
  else Result := Stuff(Result,1,1,AnsiUpperCase(Copy(Result,1,1)));
end;

function MoneyInWords(Sum: Extended; MainCur, SecCur: TCurrencyNames; MainGender: TGenders;
                      Decimals: Integer {$IFDEF POLARIS_D4}= 2{$ENDIF}): string;
var
  n1: Integer;
begin
  Result := NumInWords(Sum, MainGender)+MainCur[TestNum(Trunc(Sum))];
  if Decimals > 0 then begin
    n1 := Round(Frac(Sum)*RaisePower(10,Decimals));
    Result := Result + crBlank + FormatFloat(StringOfChar('0', Decimals), n1) +
              crBlank + SecCur[TestNum(n1)];
  end;
end;

function NumSomething(Num: Integer; Something: String; IncludeNum: Boolean): String;
{$IFDEF POLARIS_D4}overload;{$ENDIF}
begin
  Result := ExtractWord(TestNum(Num)+1, Something, [',']);
  if IncludeNum then
    Result := IntToStr(Num) + crBlank + Result;
end;
{$IFDEF POLARIS_D4}
function NumSomething(Num: Integer; Something: TCurrencyNames; IncludeNum: Boolean): String; overload;
begin
  Result := Something[TestNum(Trunc(Num))];
  if IncludeNum then
    Result := IntToStr(Num) + crBlank + Result;
end;
{$ENDIF}

function IncCycle(var Value: Integer;const Offset, MinValue, MaxValue: Integer): Integer;
var
  Offs: Integer;
begin
  Result := 0;
  if MaxValue < MinValue then exit;
  Offs := Offset;
  repeat
    Inc(Value, Offs);
    if Value > MaxValue then begin
      Inc(Result);
      Offs := Value-MaxValue;
      Value := MinValue-1;
    end else
      if Value < MinValue then begin
        Dec(Result);
        Offs := Value-MinValue;
        Value := MaxValue+1;
      end;
  until (Value>=MinValue) and (Value<=MaxValue);
end;

function Sign;
begin
  if Value = 0 then Result := 0 else
    if Value < 0 then Result := -1 else Result := 1;
end;

function RaisePower(Value,Power: Integer): Integer;
var i: Integer;
begin
  Result := 1;
  for i := 1 to Power do Result := Result*Value;
end;

function fPower(Value, Power: Extended): Extended;
begin
  Result := exp(Power*ln(Value));
end;

function fRound(X: Extended; Precision: Integer): Extended;
begin
  Result := Int(X*fPower(10,Precision)+Sign(X)*0.500000001)*fPower(10,-Precision);
end;

function SoftRound(X: Extended; Precision: Integer): Extended;
begin
  Result := fRound(X, Precision);
  if Result = 0 then Result := X;
end;

function DivZero(X, Y: Extended; DefValue: Extended {$IFDEF POLARIS_D4}=0{$ENDIF}): Extended;
begin
  if Y = 0 then Result := DefValue else Result := X/Y;
end;

function GetOSUser: string;
{$ifdef mswindows}
var
  sbuf: {$IFDEF POLARIS_D12}PChar{$ELSE}PAnsiChar{$ENDIF};
  i: DWORD;
  n: Integer;
begin
  n := 80;
  i := n;
  GetMem(sbuf, n);
  try
    GetUserName(sbuf, i);
    Result := sbuf;
  finally
    FreeMem(sbuf, n);
  end;
{$endif}
{$ifdef linux}
begin
  Result := getenv('USERNAME');
{$endif}
end;

function GetCompName: string;
{$ifdef mswindows}
var
  sbuf: {$IFDEF POLARIS_D12}PChar{$ELSE}PAnsiChar{$ENDIF};
  i: DWORD;
  n: Integer;
begin
  n := MAX_COMPUTERNAME_LENGTH+1;
  i := n;
  GetMem(sbuf, n);
  try
    GetComputerName(sbuf, i);
    Result := sbuf;
  finally
    FreeMem(sbuf, n);
  end;
{$endif}
{$ifdef linux}
begin
  Result := getenv('HOSTNAME');
{$endif}
end;

function GetModulePath(hModule: THandle): string;
var
{$ifdef mswindows}
  p: PChar;
{$endif}
{$ifdef linux}
  Addr: Pointer;
  Info: TDLInfo;
{$endif}
begin
  Result := crEmptyStr;
  {$ifdef mswindows}
  if (hModule = 0) and (GetCurrentProcess < THandle(MaxInt)*2) then
    hModule := GetCurrentProcess + 1;
  GetMem(p, MAX_PATH + 1);
  try
    GetModuleFileName(hModule, p, MAX_PATH);
    Result := p;
  finally
    FreeMem(p, MAX_PATH + 1);
  end;
  {$endif}
  {$ifdef linux}
  dlerror;
  if (hModule = MainInstance) or (hModule = 0)
    then Addr := ExeBaseAddress
    else Addr := dlsym(Pointer(hModule), '_DYNAMIC');
  if (Addr <> nil) and (dlerror = nil) and (dladdr(Addr, Info) <> 0)
    then Result := Info.dli_fname;
  {$endif}
end;

{$ifdef linux}
function GetModulePath(hModule: Pointer): string;
begin
  Result := GetModulePath(DWORD(hModule));
end;
{$endif}

function GetGlobalConfigDir(): string;
{$ifdef mswindows}
var
  p: PChar;
{$endif}
begin
  {$ifdef mswindows}
  GetMem(p, MAX_PATH + 1);
  try
    GetWindowsDirectory(p, MAX_PATH);
    Result := p;
  finally
    FreeMem(p, MAX_PATH + 1);
  end;
  {$endif}
  {$ifdef linux}
  Result := '/etc'
  {$endif}
end;

function GetLocalConfigDir(): string;
{$IFNDEF POLARIS_D6}
var
  Buffer: array[0..4095] of Char;
{$ENDIF}
begin
  {$ifdef mswindows}
{$IFDEF POLARIS_D6}
  Result := GetEnvironmentVariable('HOMEPATH');
{$ELSE}
  SetString(Result, Buffer, GetEnvironmentVariable('HOMEPATH', Buffer, SizeOf(Buffer)))
{$ENDIF}
  {$endif}
  {$ifdef linux}
  Result := getenv('HOME');
  {$endif}
end;

{$IFDEF MSWINDOWS}

procedure SetOSTime(D: TDateTime);
var
  SYSTIME: TSystemTime;
  TIMEZONE: TTimeZoneInformation;
  t: DWORD;
begin
  with SYSTIME do begin
    t := GetTimeZoneInformation(TIMEZONE);
    if t = $FFFFFFFF then
{$IFDEF POLARIS_D6}
      RaiseLastOSError;
{$ELSE}
      RaiseLastWin32Error;
{$ENDIF}
    D := D + TIMEZONE.Bias / 24 / 60;
    if t = TIME_ZONE_ID_STANDARD then
      D := D + TIMEZONE.StandardBias / 24 / 60
    else
    if t = TIME_ZONE_ID_DAYLIGHT then
      D := D + TIMEZONE.DaylightBias / 24 / 60;
    DecodeDate(D, wYear, wMonth, wDay);
    DecodeTime(D, wHour, wMinute, wSecond, wMilliseconds);
  end;
  if not SetSystemTime(SYSTIME) then
{$IFDEF POLARIS_D6}
    RaiseLastOSError;
{$ELSE}
    RaiseLastWin32Error;
{$ENDIF}
end;

procedure ExitOS(Flag: UINT);
var
  th: THandle;
  TP: TTokenPrivileges;
  dw: DWORD;
begin
  TP.PrivilegeCount := 1;
  TP.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;
  if (Win32Platform = VER_PLATFORM_WIN32_NT) and
     (not OpenProcessToken(GetCurrentProcess, TOKEN_ADJUST_PRIVILEGES OR TOKEN_QUERY, th) or
      not LookupPrivilegeValue(nil, 'SeShutdownPrivilege', TP.Privileges[0].Luid) or
      not AdjustTokenPrivileges(th, False, TP, SizeOf(TP), TP, dw)) then
{$IFDEF POLARIS_D6}
    RaiseLastOSError;
{$ELSE}
    RaiseLastWin32Error;
{$ENDIF}
  if not ExitWindowsEx(Flag, 0) then
{$IFDEF POLARIS_D6}
    RaiseLastOSError;
{$ELSE}
    RaiseLastWin32Error;
{$ENDIF}
end;

{$endif mswindows}

function GetVersionInfo(VerList: TStrings; FileName: string): Boolean;
{$ifdef mswindows}
const
  nItems = 11;
  ItemName: array[0..nItems] of String = (
    srVIComp, srVIFDesc, srVIFVer, srVIInt, srVICprt, srVITM,
    srVIFile, srVIProd, srVIProdV, srVIComm, srVIEmail, srVIHome);
var
  VersionBuffer, TempText: Pointer;
  VersionSize, Dummy: DWord;
  PSize, I: Integer;
  s: string;
{$endif mswindows}
{$ifdef linux}
var
  f: pbfd;
  bfdformat: bfd_format;
  sect: pbfd_section;
  p: Pointer;
  sz: Integer;
{$endif linux}
begin
  Result := False;
  if not Assigned(VerList) then Exit;
  if FileName = crEmptyStr then FileName := ParamStr(0);
{$ifdef mswindows}
  VerList.Clear;
  VersionSize := GetFileVersionInfoSize(PChar(FileName),Dummy);
  if VersionSize <> 0 then begin
    PSize := VersionSize;
    GetMem(VersionBuffer,PSize);
    try
      if GetFileVersionInfo(PChar(FileName),Dummy,VersionSize,VersionBuffer) and
         VerQueryValue(VersionBuffer,'\VarFileInfo\Translation',TempText,VersionSize)
      then begin
        s := '\StringfileInfo\'+IntToHex(PWord(TempText)^,4)+IntToHex(PWord(Integer(TempText)+2)^,4)+'\';
        for I := 0 to nItems do
          if VerQueryValue(VersionBuffer,
                           PChar(s+ItemName[I]),
                           TempText,VersionSize)
          then VerList.Values[ItemName[I]] := PChar(TempText);
      end;
    finally
      FreeMem(VersionBuffer,PSize);
    end;
  end;
{$endif mswindows}
{$ifdef linux}
  bfd_init();

  f := bfd_openr(PChar(FileName), nil);
  if (f = nil) then Exit;

  bfdformat := bfd_object;
  if not bfd_check_format(f, DWORD(bfdformat)) then Exit;

  sect := bfd_get_section_by_name(f, '.polaris.versioninfo');
  if sect <> nil then begin
    sz := bfd_section_size(sect);
    if sz > 0 then begin
      GetMem(p, sz + 1);
      PByte(Integer(p) + sz)^ := 0;
      if (bfd_get_section_contents(f, sect, p, 0, sz)) then begin
        VerList.Text := PChar(p);
        Result := True;
      end;
      FreeMem(p);
    end
  end;

  bfd_close(f);

{$endif linux}
end;

function GetFileVersion(FileName: string): string;
var
{$ifdef mswindows}
  VersionBuffer: Pointer;
  VersionSize, Dummy: DWORD;
  PSize: Integer;
  PntVSFixedFileInfo: PVSFixedFileInfo;
{$endif mswindows}
{$ifdef linux}
  sl: TStringList;
{$endif linux}
begin
  Result := crEmptyStr;
  if FileName = crEmptyStr then FileName := ParamStr(0);
{$ifdef mswindows}
  VersionSize := GetFileVersionInfoSize(PChar(FileName), Dummy);
  if VersionSize <> 0 then begin
    PSize := VersionSize;
    GetMem(VersionBuffer, PSize);
    try
      if GetFileVersionInfo(PChar(FileName), Dummy, VersionSize, VersionBuffer) and
         VerQueryValue(VersionBuffer, '\', Pointer(PntVSFixedFileInfo), VersionSize)
      then begin
        Dummy := PntVSFixedFileInfo^.dwFileVersionMS;
        Result := IntToStr(Dummy shr 16) + '.' + IntToStr(Dummy and $FFFF);
        Dummy := PntVSFixedFileInfo^.dwFileVersionLS;
        Result := Result + '.' + IntToStr(Dummy shr 16) + '.' + IntToStr(Dummy and $FFFF);
      end;
    finally
      FreeMem(VersionBuffer, PSize);
    end;
  end;
{$endif mswindows}
{$ifdef linux}
  sl := TStringList.Create();
  try
    if GetVersionInfo(sl, FileName) then
      Result := sl.Values['FileVersion'];
  finally
    sl.Free();
  end;
{$endif linux}
end;

{$ifdef mswindows}

function RegReadValue(Reg: TRegistry; Name: string; Default: string): string;
{$IFDEF POLARIS_D4}overload;{$ENDIF}
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadString(Name)
  else Result := Default;
end;
{$IFDEF POLARIS_D4}
function RegReadValue(Reg: TRegistry; Name: string; Default: Integer): Integer; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadInteger(Name)
  else Result := Default;
end;
function RegReadValue(Reg: TRegistry; Name: string; Default: Extended): Extended; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadFloat(Name)
  else Result := Default;
end;
function RegReadValue(Reg: TRegistry; Name: string; Default: Boolean): Boolean; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadBool(Name)
  else Result := Default;
end;
function RegReadValue(Reg: TRegistry; Name: string; Default: TDateTime): TDateTime; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadDateTime(Name)
  else Result := Default;
end;
{$ENDIF} // POLARIS_D4
{$endif mswindows}

{$IFDEF POLARIS_D4}
function RegReadValue(Reg: TrRegIni; Name: string; Default: string = crEmptyStr): string; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadString(Name)
  else Result := Default;
end;
function RegReadValue(Reg: TrRegIni; Name: string; Default: Integer = 0): Integer; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadInteger(Name)
  else Result := Default;
end;
function RegReadValue(Reg: TrRegIni; Name: string; Default: Extended = 0): Extended; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadFloat(Name)
  else Result := Default;
end;
function RegReadValue(Reg: TrRegIni; Name: string; Default: Boolean = False): Boolean; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadBool(Name)
  else Result := Default;
end;
function RegReadValue(Reg: TrRegIni; Name: string; Default: TDateTime = 0): TDateTime; overload;
begin
  if Reg.ValueExists(Name) then Result := Reg.ReadDateTime(Name)
  else Result := Default;
end;

{$ENDIF} // POLARIS_D4

function SeparateRegKey(Source: string; var TargetRoot: HKEY; var TargetKey: string): Boolean;
var
  i: Integer;
  s: String;
begin
  TargetRoot := HKEY_CURRENT_USER;
  TargetKey := Source;
  Result := False;
  if (Length(Source) < 2) or (Source[1] = '\') then Exit;
  i := pos('\', Source);
  if i = 0 then Exit;
  s := Copy(Source, 1, i - 1);
  if (s = 'HKEY_CLASSES_ROOT') or     (s = 'HKCR') then TargetRoot := HKEY_CLASSES_ROOT
  else
  if (s = 'HKEY_CURRENT_USER') or     (s = 'HKCU') then TargetRoot := HKEY_CURRENT_USER
  else
  if (s = 'HKEY_LOCAL_MACHINE') or    (s = 'HKLM') then TargetRoot := HKEY_LOCAL_MACHINE
  else
  if (s = 'HKEY_USERS') or            (s = 'HKUS') then TargetRoot := HKEY_USERS
  else
  if (s = 'HKEY_PERFORMANCE_DATA') or (s = 'HKPD') then TargetRoot := HKEY_PERFORMANCE_DATA
  else
  if (s = 'HKEY_CURRENT_CONFIG') or   (s = 'HKCC') then TargetRoot := HKEY_CURRENT_CONFIG
  else
  if (s = 'HKEY_DYN_DATA') or         (s = 'HKDD') then TargetRoot := HKEY_DYN_DATA
  else Exit;
  TargetKey := Copy(Source, i, Length(Source)-i+1);
  Result := True;
end;

function RegKeyToKeyName(aRegKey: HKEY): string;
begin
  case aRegKey of
    HKEY_CLASSES_ROOT:     Result := 'HKEY_CLASSES_ROOT';
    HKEY_CURRENT_USER:     Result := 'HKEY_CURRENT_USER';
    HKEY_LOCAL_MACHINE:    Result := 'HKEY_LOCAL_MACHINE';
    HKEY_USERS:            Result := 'HKEY_USERS';
    HKEY_PERFORMANCE_DATA: Result := 'HKEY_PERFORMANCE_DATA';
    HKEY_CURRENT_CONFIG:   Result := 'HKEY_CURRENT_CONFIG';
    HKEY_DYN_DATA:         Result := 'HKEY_DYN_DATA';
    else Result := crEmptyStr;
  end;
end;

function SeparateVersion(Version: String; var MS, LS: DWORD): Boolean;
var
  i, WC: Integer;
  s: String;
  V: array [1..4] of DWORD;
begin
  Result := False;
  S := crEmptyStr;
  MS := 0;
  LS := 0;
  WC := WordCount(Version, [crVersionSep]);
  if WC > High(V) then exit;
  for i := 1 to High(V) do begin
    if i <= WC then begin
      s := ExtractDelimited(i, Version, [crVersionSep]);
      try
        V[i] := StrToInt(s);
      except
        exit;
      end;
    end else
      V[i] := 0;
  end;
  MS := V[1] shl 16 + V[2];
  LS := V[3] shl 16 + V[4];
  Result := True;
end;

function CompareVersion(Ver1, Ver2: String): Integer;
var
  MS1, LS1, MS2, LS2: DWORD;
begin
  if not SeparateVersion(Ver1, MS1, LS1) or
     not SeparateVersion(Ver2, MS2, LS2)
    then raise Exception.Create(srVersionFormatError);
  Result := Sign(Integer(MS1) - Integer(MS2));
  if (Result = 0) then
    Result := Sign(Integer(LS1) - Integer(LS2));
end;

function CompareVar(Var1, Var2: Variant): Integer;
type
  pb = ^Byte;
var
  i, size1, size2: Integer;
  p1, p2: pb;
begin
  if VarIsEmpty(Var1) then
    if VarIsEmpty(Var2) then Result := 0 else Result := -1
  else
  if VarIsNull(Var1) then
    if VarIsEmpty(Var2) then Result := 1
    else if VarIsNull(Var2) then Result := 0 else Result := -1
  else
  if VarIsEmpty(Var2) or VarIsNull(Var2) then Result := 1
  else
    if VarIsArray(Var1) then begin
      //--- compare arrays
      if not VarIsArray(Var2) then begin
        Result := 1;
        exit;
      end;
      // calculate size of arrays
      p1 := VarArrayLock(Var1);
      try
        size1 := 0;
        for i := 0 to TVarData(Var1).VArray.DimCount-1 do
          Inc(size1, TVarData(Var1).VArray.Bounds[i].ElementCount *
                     TVarData(Var1).VArray.ElementSize);
        p2 := VarArrayLock(Var2);
        try
          size2 := 0;
          for i := 0 to TVarData(Var2).VArray.DimCount-1 do
            Inc(size2, TVarData(Var2).VArray.Bounds[i].ElementCount *
                       TVarData(Var2).VArray.ElementSize);
          if (size1 <> size2) then Result := Sign(size1 - size2)
          else begin
            // compare elements
            Result := 0;
            for i := 0 to size1-1 do begin
              Result := Sign(pb(Integer(p1)+i)^ - pb(Integer(p2)+i)^);
              if Result <> 0 then exit;
            end;
          end;
        finally
          VarArrayUnlock(Var2);
        end;
      finally
        VarArrayUnlock(Var1);
      end;
    end else
      //--- compare single values
      if Var1 < Var2 then Result := -1
      else if Var1 = Var2 then Result := 0 else Result := 1;
end;

function AssignList(Dest, Source: TList): Integer;
begin
  Result := 0;
  if not Assigned(Source) then Exit;
  if not Assigned(Dest) then Exit;
  Dest.Clear;
  for Result := 0 to Source.Count - 1 do
    Dest.Add(Pointer(Source[Result]));
  Result := Source.Count;
end;

procedure CopyObject(Source, Dest: TObject);
var
  i: Integer;
  TypeData: PTypeData;
  Props: PPropList;
begin
  if (Source <> nil) and (Dest <> nil) then begin
    TypeData := GetTypeData(Source.ClassInfo);
    if (TypeData <> nil) and (TypeData^.PropCount > 0) then begin
      GetMem(Props, TypeData^.PropCount * sizeof(Pointer));
      try
        GetPropInfos(Source.ClassInfo, Props);
        for i := 0 to TypeData^.PropCount - 1 do
          if Props^[i]^.Name <> 'Name' then begin
            case Props^[i]^.PropType^.Kind of
              tkInteger:
                SetOrdProp(Dest, Props^[i], GetOrdProp(Source, Props^[i]));
              tkString, tkLString:
                SetStrProp(Dest, Props^[i], GetStrProp(Source, Props^[i]));
              tkFloat:
                SetFloatProp(Dest, Props^[i], GetFloatProp(Source, Props^[i]));
              tkMethod:
                SetMethodProp(Dest, Props^[i], GetMethodProp(Source, Props^[i]));
{$IFDEF POLARIS_D5}
              tkEnumeration:
                SetEnumProp(Dest, Props^[i], GetEnumProp(Source, Props^[i]));
              tkClass:
                SetObjectProp(Dest, Props^[i], GetObjectProp(Source, Props^[i]));
{$ENDIF}
            end;
          end;
      finally
        Freemem(Props);
      end;
    end;
  end;
end;

function IIF(Switch: Boolean; Var1,Var2: Variant): Variant;
{$IFDEF POLARIS_D4}overload;{$ENDIF}
begin
  if Switch then Result := Var1 else Result := Var2;
end;
{$IFDEF POLARIS_D4}
function IIF(Switch: Boolean; Var1,Var2: String): String; overload;
begin
  if Switch then Result := Var1 else Result := Var2;
end;
{$ENDIF}

function GetRealIndex(Items: TStrings; FirstIndex: Integer; S: string): Integer;
var i: Integer;
begin
  Result := -1;
  if Items[FirstIndex] <> S then
    for i:=Max(FirstIndex,0) to Items.Count-1 do
      if Items[i] = S then begin
        Result := i;
        exit;
      end
      else
  else Result := FirstIndex;
end;

function ReadEnumValues(Items: TStrings; Values: TStrings; ItemName, ItemCountName: string;
                        FirstIndex: Integer): Integer;
var
  i, n: Integer;
  s: string;
begin
  n := -1;
  if Length(ItemCountName) > 0 then
    n := StrToIntDef(Items.Values[ItemCountName], -1);
  Result := 0;
  i := FirstIndex;
  if n <> 0 then
    repeat
      s := Items.Values[ItemName + IntToStr(i)];
      if Length(s) > 0 then begin
        Values.Add(s);
        Inc(Result);
      end;
      Inc(i);
    until ((n = -1) and (Length(s) = 0)) or ((n <> -1) and (i-FirstIndex >= n));
end;

function InheritsFromOneOf(const aClass: TClass; aClasses: array of TClass): Boolean;
var
  i: Integer;
begin
  Result := False;
  for i := 0 to High(aClasses) do begin
    Result := aClass.InheritsFrom(aClasses[i]);
    if Result then Break;
  end;
end;

function InheritsFromOneOf(const aObject: TObject; aClasses: array of TClass): Boolean;
begin
  Result := InheritsFromOneOf(aObject.ClassType, aClasses);
end;

{$ifdef mswindows}
function LoadFromReg(Reg: TRegistry; RegKey, FKey: string; Default: string {$IFDEF POLARIS_D4}= crEmptyStr{$ENDIF}): string;
var
  b: boolean;
begin
  Result := EmptyStr;
  b := Assigned(Reg);
  if not b then Reg := TRegistry.Create;
  try
    with Reg do begin
      if OpenKey(RegKey, False) then
      try
        Result := RegReadValue(Reg, FKey, Default);
      finally
        CloseKey;
      end;
    end;
  finally
    if not b then Reg.Free;
  end;
end;

procedure SaveToReg(Reg: TRegistry; RegKey, FKey: string; Value: Integer);
var
  b: boolean;
begin
  b := Assigned(Reg);
  if not b then Reg := TRegistry.Create;
  try
    with Reg do begin
      OpenKey(RegKey, True);
      try
        WriteInteger(FKey, Value);
      finally
        CloseKey;
      end;
    end;
  finally
    if not b then Reg.Free;
  end;
end;

{$IFDEF POLARIS_D4}
procedure SaveToReg(Reg: TRegistry; RegKey, FKey: string; Value: string);
var
  b: boolean;
begin
  b := Assigned(Reg);
  if not b then Reg := TRegistry.Create;
  try
    with Reg do begin
      OpenKey(RegKey, True);
      try
        WriteString(FKey, Value);
      finally
        CloseKey;
      end;
    end;
  finally
    if not b then Reg.Free;
  end;
end;
{$ENDIF}
{$endif mswindows}

initialization
  RHandles := TList.Create;
  RHandles.Add(Pointer(HInstance));
{$ifdef mswindows}
  with TRegistry.Create do
    try
      use_wine := KeyExists('Software\Wine');
    finally
      Free;
    end;
{$endif mswindows}
finalization
  RHandles.Free;
end.

