(*====================================================================
    File rdbclientutils.pas

    Components library for Delphi, Kylix and BCB++.
    Functions, related with TClientDataSet.

    Copyright  2007-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

======================================================================
$Id: rdbclientutils.pas,v 1.8 2008/09/05 09:00:45 savchenko Exp $
======================================================================*)
unit rdbclientutils;

interface

uses DB, Provider, DBClient, rUtils, RxStrUtils;

type
  TrFieldAttribute = (tfaDisplayLabel, tfaDisplayFormat, tfaEditFormat, tfaAggregate, tfaInVisibleFields);

var
  GetProviderProc: function(aCDS: TDataSet): TDataSetProvider;

function GetNativeDataSet(aDS: TDataSet): TDataSet;

//   aFieldAttribute 
//  aDataSet   aValueList 
//  aDelim
procedure SetFieldsAttribute(aDataSet: TDataSet; aFieldAttribute: TrFieldAttribute; aValueList: string; aDelim: TCharSet = [';']);

implementation

function GetNativeDataSet(aDS: TDataSet): TDataSet;
var
  pr: TDataSetProvider;
  DataSet: TDataSet;
begin
  Result := nil;
  if aDS is TClientDataSet then begin
    if Assigned(TClientDataSet(aDS).CloneSource)
      then DataSet := TClientDataSet(aDS).CloneSource
      else DataSet := aDS;
    pr := nil;
    if Assigned(GetProviderProc) then
      pr := GetProviderProc(aDS);
    if not Assigned(pr) then
      if Assigned(DataSet.Owner) then
        pr := TDataSetProvider(DataSet.Owner.FindComponent(TClientDataSet(DataSet).ProviderName));
    if Assigned(pr) and Assigned(pr.DataSet) then
      Result := pr.DataSet;
  end
  else
    Result := aDS;
end;

procedure SetFieldsAttribute(aDataSet: TDataSet; aFieldAttribute: TrFieldAttribute; aValueList: string; aDelim: TCharSet = [';']);
var
  i, k: integer;
  Field: TField;
  Value: String;
begin
  with aDataSet do begin
    k := GetDelimCount(aValueList, aDelim) + 1;
    i := FieldCount;
    if i < k then k := i;
    for i := 1 to k do begin
      Field := Fields[i - 1];
      Value := ExtractDelimited(i, aValueList, aDelim);
      case aFieldAttribute of
        tfaDisplayLabel: Field.DisplayLabel := Value;
        tfaDisplayFormat: if Field is TNumericField then TNumericField(Field).DisplayFormat := Value;
        tfaEditFormat: if Field is TNumericField then TNumericField(Field).EditFormat := Value;
        tfaAggregate: if aDataSet is TClientDataSet then
                        with TClientDataSet(aDataSet).Aggregates.Add do begin
                          AggregateName := Field.Name + '_';
                          Expression := Value + '(' + Field.DisplayName + ')';
                          Active := True;
                        end;
        tfaInVisibleFields: if (Value <> crEmptyStr) and Assigned(FindField(Value)) then
                              FieldByName(Value).Visible := False;
      end;
    end;
  end;
end;

end.
 